#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""RNAkb (previous Gromacs) utils.

A module with different functions needed for Gromacs/RNAkb merriage.

Marcin Magnus
Albert Bogdanowicz

(1) prepare groups and then (2) mdp score file.
"""

import re
import os
import argparse

LIB_PATH = os.path.dirname(os.path.dirname(os.path.abspath(__file__))) + os.sep
VERBOSE = False
MDP_TEMPLATE = 'data/score_rnakb_orig.mdp'

GROMACS_ALLOWED_5 = ("H5T", "O5'", "C5'", "H5'1", "H5'2", "C4'", "H4'",
        "O4'", "C1'", "H1'", "N9", "C8", "H8", "N7", "C5", "C6", "N6", "H61",
        "H62", "N1", "C2", "H2", "N3", "C4", "C3'", "H3'", "C2'", "H2'1",
        "O2'", "HO'2", "O3'",
        # new atoms
        "O2", "O4", "O6", "N2", "N4",
        'OP1', 'OP2', 'OP3', 'H21', 'H22', "H2'", "H5'", "HO5'", "H5''", "HO2'",
        'P', 'H3', 'H1', 'H6', 'H5', 'H42', 'H41',
        )

GROMACS_ALLOWED_MIDDLE = ("P", "O1P", "O2P", "O5'", "C5'", "H5'1", "H5'2",
       "C4'", "H4'", "O4'", "C1'", "H1'", "N9", "C8", "H8", "N7", "C5", "C6",
        "N6", "H61", "H62", "N1", "C2", "H2", "N3", "C4", "C3'", "H3'", "C2'",
        "H2'1", "O2'", "HO'2", "O3'",
        # new atoms
        "O2", "O4", "O6", "N2", "N4",
        'OP1', 'OP2', 'OP3', 'H21', 'H22', 'O6', "H2'", "H5'", "HO5'", "H5''", "HO2'",
        'P', 'H3', 'H1', 'H6', 'H5', 'H42', 'H41',
        )

GROMACS_REQUIRED_GROUPS = ("aP", "aC4s", "aC2", "aC4", "aC6",
                           "uP", "uC4s", "uC2", "uC4", "uC6",
                           "gP", "gC4s", "gC2", "gC4", "gC6",
                           "cP", "cC4s", "cC2", "cC4", "cC6",
                           "RNA_5pt", "other"
        )


def get_res_num(line):
    """Extract residue number from a line of PDB file

    Arguments:
      * line = ATOM line from a PDB file

    Output:
      * residue number as an integer
    """
    return int(''.join([x for x in line[22:27] if x.isdigit()]))


def get_res_code(line):
    """Get residue code from a line of a PDB file
    """
    return line[17:20]

def set_res_code(line, code):
    """Set residue code from a line of a PDB file
    """
    return line[:17] + code.rjust(3) + line[20:]

def make_rna_gromacs_ready(pdb_string, verbose=VERBOSE):
    """GROMACS has some special requirements for PDB files.

    Arguments:
      * pdb_string = contents of PDB file as a string

    Output:
      * new PDB returned as a string

    (!!!) # hmm... [ RA5 ] will not be detected based on it (!?)
    Hmm.. becase it detects if the structure is already prepared.
    """
    #pdb_string = resname_3to1(pdb_string)
    #pdb_string = remove_hetatms(pdb_string)
    result = []
    pdb_lines = pdb_string.split('\n')

    # find smallest residue number
    min_res = min(list(map(get_res_num,
        [l for l in pdb_lines if l.startswith('ATOM')])))
    max_res = max(list(map(get_res_num,
        [l for l in pdb_lines if l.startswith('ATOM')])))


    for l in pdb_lines:
        if l.startswith('ATOM') and l[19] in ('A', 'U', 'C', 'G'): # hmm... [ RA5 ] will not be detected based on it (!?)
            res = get_res_code(l)        
            if res.startswith('R') and res.endswith('5') : # it's RX5 file so skip fixing
                if verbose:
                    print('-- already gromacs ready')
                return pdb_string

            l = l.replace('*', '\'')
            l = l.replace('O1P', 'OP1')
            l = l.replace('O2P', 'OP2')

            res_num = get_res_num(l)
            atom_type = l.split()[2].strip()
            # remove P OP1 OP2
            if res_num == min_res and atom_type == 'P':
                continue
            if res_num == min_res and atom_type == 'OP1':
                continue
            if res_num == min_res and atom_type == 'OP2':
                continue

            # convert G -> RG5, RG3
            if res_num == min_res: # RG5
                l = set_res_code(l, 'R' + get_res_code(l).strip() + '5')
            elif res_num == max_res: # RG3
                l = set_res_code(l, 'R' + get_res_code(l).strip() + '3')
            else:
                l = set_res_code(l, ' R' + get_res_code(l).strip())

            if res_num == min_res:
                if atom_type in GROMACS_ALLOWED_5:
                    result.append(l)
                else:
                    print(('Wrong start line: ', l, atom_type))
            else:
                if atom_type in GROMACS_ALLOWED_MIDDLE:
                    result.append(l)
                else:
                    print(('Wrong middle line: ', l, atom_type))
        else:
            result.append(l)
    return '\n'.join(result)

def make_rna_rnakb_ready(pdb_string, verbose=VERBOSE):
    """RNAkb read (difference between this function and 
    make_rna_gromacs_ready is ignoring R5U etc. RNAkb does not treat
    them differently so there is no point to distinguish them.

    Arguments:
      * pdb_string = contents of PDB file as a string

    Output:
      * new PDB returned as a string
    """
    #pdb_string = resname_3to1(pdb_string)
    #pdb_string = remove_hetatms(pdb_string)
    result = []
    pdb_lines = pdb_string.split('\n')

    # find smallest residue number
    min_res = min(list(map(get_res_num,
        [l for l in pdb_lines if l.startswith('ATOM')])))
    max_res = max(list(map(get_res_num,
        [l for l in pdb_lines if l.startswith('ATOM')])))

    for l in pdb_lines:
        if l.startswith('ATOM'):# and l[19] in ('A', 'U', 'C', 'G'):
            res = get_res_code(l)
            #if res.startswith('R') and res.endswith('5') : # it's RX5 file so skip fixing
            #    if verbose:
            #        print '-- already gromacs ready'
            #    return pdb_string

            l = l.replace('*', '\'')
            l = l.replace('O1P', 'OP1')
            l = l.replace('O2P', 'OP2')

            res_num = get_res_num(l)
            atom_type = l.split()[2].strip()
            # remove P OP1 OP2
            if res_num == min_res and atom_type == 'P':
                continue
            if res_num == min_res and atom_type == 'OP1':
                continue
            if res_num == min_res and atom_type == 'OP2':
                continue

            # convert G -> RG5, RG3
            #if res_num == min_res: # RG5
            #    l = set_res_code(l, 'R' + get_res_code(l).strip() + '5')
            #elif res_num == max_res: # RG3
            #    l = set_res_code(l, 'R' + get_res_code(l).strip() + '3')
            #else:
            l = set_res_code(l, ' R' + get_res_code(l).strip().replace('R','').replace('3','').replace('5','')) # 
            if res_num == min_res:
                if atom_type in GROMACS_ALLOWED_5:
                    result.append(l)
                else:
                    print(('Wrong start line: ', l, atom_type))
            else:
                if atom_type in GROMACS_ALLOWED_MIDDLE:
                    result.append(l)
                else:
                    print(('Wrong middle line: ', l, atom_type))
        else: # keep TER, etc.
            result.append(l)
    return '\n'.join(result)

def fix_gromacs_gro(path, verbose=False):
    """It's probably a bug in GROMACS, but box coordinates in gro files are
    not always separated by spaces. This function guesses how it should be
    separated and inserts spaces.

    Arguments:
      * path = path to gro file

    Output:
      * file is overwritten with a corrected one
    """
    f = open(path)
    gro = f.read()
    f.close()
    gro_lines = gro.split('\n')
    last_line = gro_lines[-2]

    # check if there are a space
    if last_line.find(' ') == -1:
        dots = [i.start() for i in re.finditer('\\.', last_line)]
        # next 4 lines are a guess, I hope it works
        digits = len(last_line[dots[2]:])
        box = [last_line[:dots[0] + digits],
            last_line[dots[0] + digits:dots[1] + digits], last_line[dots[1] + digits:]]
        gro_lines = gro_lines[:-2]
        gro_lines.append(' '.join(box))
        gro_lines.append('')
        f = open(path, 'w')
        f.write('\n'.join(gro_lines))
        f.close()


def fix_gromacs_ndx(path):
    """Sometimes, GROMACS index has some atoms in more than one group, or
    doesn't have all the groups grompp requires. This function fixes that.

    Arguments:
      * path = path to index file

    Output:
      * index is overwritten with a corrected one
    """
    f = open(path)
    index = f.read()
    f.close()
    # split file into system group and the rest
    system_group = index.split('[ System ]')[1].split('[')[0]
    other_groups = ['[' + i for i in
            index.split('[ System ]')[1].split('[')[1:]]
    # remove duplicate numbers
    taken_atoms = []
    for g in range(len(other_groups)):
        header = other_groups[g].split('\n')[0]
        group = other_groups[g].split('\n')[1].split()
        group = [a for a in group if a not in taken_atoms]
        taken_atoms.extend(group)
        other_groups[g] = header + '\n' + ' '.join(group) + '\n'
    # build result, part 1
    result = ['[ System ]' + system_group]
    result.extend(other_groups)
    # add missing groups, leave them empty
    headers = [g.split('\n')[0][2:-2] for g in other_groups]
    missing_headers = [h for h in GROMACS_REQUIRED_GROUPS if h not in headers]
    result.extend(['[ %s ]\n' % h for h in missing_headers])
    # write result to file
    result = ''.join(result)
    f = open(path, 'w')
    f.write(result)
    f.close()


def prepare_groups(fn, gr_fn, potential='aa', verbose=False):
    """ Prepare an index for fn file. gr_fn is a file where gtxt is saved in.

    Get seq and uniq & sort it.
    ``['RG5', 'RA', 'RA', 'RA', 'RG', 'RU', 'RA', 'RA', 'RC3'] set(['RU', 'RG', 'RC3', 'RG5', 'RA'])``

    @todo RG5 etc -- done!

    gtxt::

     del 1
     r RU & a C1'
     name 1 uC1s
     r RU & a C2
     name 2 uC2
     r RU & a C2'
     name 3 uC2s
     ...

    return, gtxt (groups_txt), energygrps . The result is saved to g_fn.
    energygrps:
    ['uP', 'uC4s', 'uC2', 'uC4', 'uC6', 'gP', 'gC4s', 'gC2', 'gC4', 'gC6', 'aP', 'aC4s', 'aC2', 'aC4', 'aC6']
    gtxt:
    RA
    del 1
    r RU & a P
    name 1 uP
    r RU & a C4s
    name 2 uC4s
    r RU & a C2
    name 3 uC2
    r RU & a C4
    [..]
    r RA & a C6
    name 15 aC6
    1|2|3|4|5|6|7|8|9|10|11|12|13|14|15
    name 16 RNA_5pt
    0 & ! 16
    name 17 other
    q
    """
    p = pf.PDBFile(pdb_path = fn) 
    seq = p.seq_from_amber_like_pdb().split()
    seq_uniq_sorted = set(seq)
    if verbose: print(('seq:', seq, seq_uniq_sorted))
    seq_rnakb_order = []
    if 'RA' in seq_uniq_sorted:
        seq_rnakb_order.append('RA')
    if 'RU' in seq_uniq_sorted:
        seq_rnakb_order.append('RU')
    if 'RG' in seq_uniq_sorted:
        seq_rnakb_order.append('RG')
    if 'RC' in seq_uniq_sorted:
        seq_rnakb_order.append('RC')
    if verbose:print(('seq_rnakb_order', seq_rnakb_order))
    
    gtxt = 'del 1\n'
    c = 1

    if potential == 'aa':
        # rg
        rg_atoms = "C3',C5,C4,C6,C8,O2',P,C2',O5',C5',C1',O3',O6,N2,N3,N1,N7,N9,C2,C4',O4',O2P,O1P".split(',')
        rg_atoms2 = ['g' + a.strip().replace("'", 's') for a in rg_atoms]
        # ru
        ug_atoms =  "C1',C2,C2',C3',C4,C4',C5,C5',C6,N1,N3,O2,O2',O3',O4,O4',O5',O1P,O2P,P".split(",")
        ug_atoms2 = ['u' + a.strip().replace("'", 's') for a in ug_atoms]
        # ag
        ag_atoms = "O3',O2',N7,N1,N3,N9,C2',O5',N6,C5',C1',C2,C6,C5,C4,O4',C4',C8,C3',P,O1P,O2P".split(',')
        ag_atoms2 = ['a' + a.strip().replace("'", 's') for a in ag_atoms]
        # cg
        cg_atoms = "C2',O2',O2P,O1P,C5',O5',C4,O2,C3',C2,O3',N4,N3,N1,P,C1',O4',C4',C5,C6".split(',')
        cg_atoms2 = ['c' + a.strip().replace("'", 's') for a in cg_atoms]
        #N1, N3, O4', C5', O3', C2', C4, C1', O5', O1P, C4', C6, C5, C2, C3', P, O2P, O2, O4, O2'
        if verbose: print(('len-s:', len(rg_atoms), len(cg_atoms), len(ag_atoms), len(ug_atoms)))
    elif potential == '5pt':
        # rg
        rg_atoms = ["P", "C4'", "C2", "C4", "C6"]
        ug_atoms = ["P", "C4'", "C2", "C4", "C6"]
        ag_atoms = ["P", "C4'", "C2", "C4", "C6"]                
        cg_atoms = ["P", "C4'", "C2", "C4", "C6"]                
        rg_atoms2 = ['g' + a.strip().replace("'", 's') for a in rg_atoms]
        ug_atoms2 = ['u' + a.strip().replace("'", 's') for a in ug_atoms]
        ag_atoms2 = ['a' + a.strip().replace("'", 's') for a in ag_atoms]
        cg_atoms2 = ['c' + a.strip().replace("'", 's') for a in cg_atoms]
    else:
        raise Exception('Unknown potential, use all or 5pt')

    energygrps = []

    for r in seq_rnakb_order:
        if r == 'RA':
            for x, y in zip(ag_atoms, ag_atoms2):
                gtxt += 'r RA & a %s\n' % x
                gtxt += 'name %i %s\n' % (c, y)
                c += 1
            energygrps.extend(ag_atoms2)

        if r == 'RU':
            for x, y in zip(ug_atoms, ug_atoms2):
                gtxt += 'r RU & a %s\n' % x
                gtxt += 'name %i %s\n' % (c, y)
                c += 1
            energygrps.extend(ug_atoms2)

        if r == 'RG':
            for x, y in zip(rg_atoms, rg_atoms2):
                gtxt += 'r RG & a %s\n' % x
                gtxt += 'name %i %s\n' % (c, y)
                c += 1
            energygrps.extend(rg_atoms2)

        if r == 'RC':
            for x, y in zip(cg_atoms, cg_atoms2):
                gtxt += 'r RC & a %s\n' % x
                gtxt += 'name %i %s\n' % (c, y)
                c += 1
            energygrps.extend(cg_atoms2)

    gtxt += '|'.join([str(x) for x in range(1,c)])
    gtxt += '\nname %i RNA_%s' % (c, potential) # @todo
    gtxt += '\n0 & ! %i' % (c)
    gtxt += '\nname %i other' % (c + 1)
    gtxt += '\nq\n'
    if verbose: print(gtxt)

    with open(gr_fn, 'w') as f:
        f.write(gtxt)

    if verbose: print(('energygrps', energygrps))
    return gtxt, energygrps, seq_rnakb_order


def format_score_mdp(mdp_out, energygrps, seq, verbose=False):
    """Get a template score mdp and replace energygrps
    (it can be generated with prepare_groups)
    and energygrp_table
    """
    # load template
    with open(LIB_PATH + 'rnakb_utils/' + MDP_TEMPLATE, 'r') as f:
        txt = f.readlines()

    with open(LIB_PATH + 'rnakb_utils/data/rnakb_all.txt', 'r') as f:
        pairs = [i.strip() for i in f.readlines()]

    nmdp = ''
    for l in txt:
        if l.startswith('energygrps'):
            l = 'energygrps               = ' + ' '.join(energygrps) + ' other'
            nmdp += l
        elif l.startswith('energygrp_table'):
            d = ''
            for x in energygrps:  # ugly :-(
                for y in energygrps:
                    s = '%s_%s' % (x, y) # Library file table_uP_aP.xvg not found in current dir nor in your GMXLIB path.
                    s_reverse = '%s_%s' % (y, x) # try: /home/mqapRNA/mqaprna_env/db/RNA_5pt_full_sc1/table_aP_uP.xvg
                    if s in pairs:
                        d += '%s ' % s.replace('_', ' ')  # '_' -> ' '
            l = 'energygrp_table          = ' + d.strip()
            nmdp += l
        elif l.startswith('energygrp_excl'):
            l = 'energygrp_excl           =  other other ' + ' other '.join(energygrps) + ' other'
            nmdp += l
        else:
            nmdp += l
    if verbose: print(nmdp)

    with open(mdp_out, 'w') as f:
        f.write(nmdp)


def get_parser():
        parser = argparse.ArgumentParser(
            description=__doc__, formatter_class=argparse.RawDescriptionHelpFormatter)

        #parser.add_argument('-', "--", help="", default="")
        parser.add_argument("-v", "--verbose",
                            action="store_true", help="be verbose")
        parser.add_argument("file", help="", default="")
        return parser

# main
if __name__ == '__main__':
    #fn = 'test_data/decoy0165_amb_clx.pdb'
    #fn = 'test_data/1duq.pdb'
    parser = get_parser()
    args = parser.parse_args()
    fn = args.file # 'test_data/cat_chunk003_2r8s_5kcycles_decoys_nonativefrags.cluster1.0_clean_noC.pdb'
    pdblines = make_rna_gromacs_ready(open(fn).read())
    print(pdblines)
    with open(fn.replace('.pdb', '_MDready.pdb'), 'w') as f:
        f.write(pdblines)



#!/usr/bin/env python

"""
usage::

  ./clarna_app.py ../../input/5k7c_clean_onechain_renumber_as_puzzle_srr.pdb
  ../../input/5k7c_clean_onechain_renumber_as_puzzle_srr.pdb
  ((((([[[[[[)))))........(.((....(]]]]]].)..(((......)))...)).)

Example:

 .. code-block:: python

    from rna_tools.utils.clarna_app import clarna_app
    if __name__ == '__main__':
        ss = '((((.[[[[[[.))))........((((.....]]]]]]...(((((....)))))..))))'
        fnCRref = clarna_app.get_ClaRNA_output_from_dot_bracket(ss)
        f = '../rna_calc_rmsd/test_data/pistol/5k7c_clean_onechain_renumber_as_puzzle_srr.pdb'
        fnCR = clarna_app.clarna_run(f, force=False)
        results = clarna_app.clarna_compare(fnCRref, fnCR)
        print results #
        #tmp_Z42i_..pdb.outCR     5k7c_clean_onechain_renumber_as_puzzle_srr.pdb.outCR      0.706      NA         0.865      NA         0.842      0.889      NA         0.000

.. warning:: Setup a bash variable: ClaRNA_play_path, and add ClaRNA_play to your $PATH (install ClaRNA_play https://gitlab.genesilico.pl/RNA/ClaRNA_play (internal GS gitlab server)"""

import argparse
import subprocess
import sys
import os
import tempfile
from rna_tools.tools.rna_convert_pseudoknot_formats.rna_pk_simrna_to_one_line import get_one_line

try:
    ClaRNA_play_path = os.environ['ClaRNA_play_path']
except:
    print('you need to set up ClaRNA_play_path in your .bashrc')

def clarna_run(fn, force=True, stacking=True, verbose=False):
    """Run ClaRNA run

    fn: str
     filename to analyze

    :return: a filename to ClaRNA output (fn + '.outCR')"""
    if verbose:
        print('stacking', stacking)
    fn_out = fn + '.outCR'
    if os.path.isfile(fn_out) and not force:
        pass
    else:
        opts = ''
        if stacking:
            opts = ' -bp+stack '
        cmd = 'clarna_run.py ' + opts + ' -ipdb ' + fn + ' > ' + fn_out
        os.system(cmd)
    if os.stat(fn_out).st_size == 0: # if file is empty also run
        cmd = 'clarna_run.py -bp+stack -ipdb ' + fn + ' > ' + fn_out
        os.system(cmd)
    return fn_out

def get_dot_bracket_from_ClaRNAoutput(inCR, verbose=False):
    """In inCR file"""
    cmd = ClaRNA_play_path + '/lib/ClaRNAwd_to_vienaSS/ClaRNAwd_output_parser_get_SS ' + inCR
    o = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    std = o.stdout.read().strip()
    if verbose: std
    return std

def clarna_compare(target_cl_fn,i_cl_fn, verbose=False):
    """Run ClaRNA compare.

    :return: a list target, fn, scores

    Scores::

        inf_all      0.706
        inf_stack -999.999 -> NA
        inf_WC       0.865
        inf_nWC   -999.999 -> NA
        SNS_WC       0.842
        PPV_WC       0.889
        SNS_nWC     NA
        PPV_nWC      0.000

    Example of the list::

       5k7c_clean_onechain_renumber_as_puzzle_srr.pdb     pistol_thrs0.50A_clust01-000001_AA.pdb
        0.642      NA         0.874      0.000      0.944      0.810      0.000      0.000s

    use ``results.split()[4]`` to get inf_WC"""

    cmd = 'clarna_compare.py -iref ' + target_cl_fn + ' -ichk ' + i_cl_fn
    if verbose: print('clarna_app::cmd', cmd)
    o = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    std = o.stdout.read().strip()
    if not std:
        raise Exception('ClaRNA output is empty, something went wrong:\n\t %s \n %s' % (cmd, std))
    if verbose: 'clarna_app::o.stderr',o.stderr.read()

    #WARNING: nWC has more than one values struc/1i6uD_M425.pdb.outCR:  ['SW_tran', 'WW_tran']
    #WARNING: nWC has more than one values struc/1i6uD_M425.pdb.outCR:  ['SW_tran', 'WW_tran']
    #1i6uD_M1.pdb.outCR                         1i6uD_M425.pdb.outCR      0.707      0.000      0.756      0.500      0.571      1.000      0.250      1.000
    return std.split('\n')[-1] # solution for this ^, keep the clarna_compare quite

def get_ClaRNA_output_from_dot_bracket(ss, temp=True, verbose=False):
    """
    Get dummy ClaRNA output out of dat bracket secondary structure (ss)

    Args:
        ss (string): secondary structure

    Return:

        a filename to ClaRNA output"""
    from rna_tools.SecondaryStructure import parse_vienna_to_pairs

    if ss.find(':') > -1:
        chain,ss = ss.split(':')
    else:
        chain = 'A'
        ss = ss

    pairs, pairs_pk = parse_vienna_to_pairs(ss, remove_gaps_in_ss=False)
    pairs += pairs_pk

    txt = 'Classifier: Clarna\n'
    txt += 'chains:  A 1 ' + str(len(ss)) + '\n'
    for bp in pairs:
        txt += '%s    %i   %s   %i          bp G C                  WW_cis   1 \n' % (chain, bp[0], chain, bp[1])
    if verbose: print(txt.strip())

    if temp:
        f = tempfile.NamedTemporaryFile()
        name = f.name
    else:
        name = 'target'

    foutCR = name + '.pdb.outCR'
    if verbose: print(foutCR)
    ft = open(foutCR, 'w')
    ft.write(txt)
    ft.close()
    return foutCR

def get_parser():
    parser =  argparse.ArgumentParser()#usage="%prog [<options>] <pdb files (test_data/*)>")
    parser.add_argument('files', help="files", nargs='+')
    parser.add_argument('-f',"--force",
                         dest="force",
                         action="store_true",
                        help="force to run ClaRNA")
    parser.add_argument('-v',"--verbose",
                         dest="verbose",
                         action="store_true",
                        help="verbose")

    return parser

if __name__ == '__main__':
    parser = get_parser()
    args = parser.parse_args()
    if len(sys.argv) == 1:
        print(parser.print_help())
        sys.exit(1)

    for f in args.files:
        print(f)
        fn_out = clarna_run(f, args.force)
        mdb = get_dot_bracket_from_ClaRNAoutput(fn_out, args.verbose) # multie line dot bracket
        db = get_one_line(mdb.split('\n'))
        print(db)



#!/usr/bin/env python3
#-*- coding: utf-8 -*-
"""RNA Sequence with secondary structure prediction methods.

This tool takes a given sequence and returns the secondary structure prediction provided by 5 different tools: RNAfold, RNAsubopt, ipknot, contextfold and centroid_fold. You must have these tools installed. You don't have to install all tools if you want to use only one of the methods.

It's easy to add more methods of your choince to this class.

Installation
~~~~~~~~~~~~~
Depends on what tools you want to use, follow the instructions below.

ContextFold
^^^^^^^^^^^^^^^^^^^^^
https://www.cs.bgu.ac.il/~negevcb/contextfold/

needs Java. Try this on Ubuntu 14-04 https://askubuntu.com/questions/521145/how-to-install-oracle-java-on-ubuntu-14-04 Single chain only!

ViennaRNA
^^^^^^^^^^^^^^
https://www.tbi.univie.ac.at/RNA/

For OSX install from the binary Installer from the page.

ipknot OSX
^^^^^^^^^^^^^
https://github.com/satoken/homebrew-rnatools

If one encounters a problem::

    [mm] Desktop$ /usr/local/opt/bin/ipknot
    dyld: Library not loaded: /usr/local/opt/glpk/lib/libglpk.40.dylib
      Referenced from: /usr/local/opt/bin/ipknot
      Reason: image not found
    [1]    51654 abort      /usr/local/opt/bin/ipknot

the solution is::

     brew install glpk # on OSX

RNA Structure
^^^^^^^^^^^^^
http://rna.urmc.rochester.edu/

Works with 5.8.1; Jun 16, 2016.

Download http://rna.urmc.rochester.edu/RNAstructureDownload.html and untar it in ``<RNA_PDB_TOOLS>/opt/RNAstructure/``; run make, the tools will be compiled in a folder ``exe``. Set up ``DATPATH`` in your bashrc to ``<RNA_PDB_TOOLS>/opt/RNAstructure/data_tables`` ``DATAPATH=/home/magnus/work/src/rna-pdb-tools/opt/RNAstructure/data_tables/`` (read more http://rna.urmc.rochester.edu/Text/Thermodynamics.html). RNAstructure can be run with SHAPE restraints, read more http://rna.urmc.rochester.edu/Text/File_Formats.html#Constraint about the format. The file format for SHAPE reactivity comprises two columns. The first column is the nucleotide number, and the second is the reactivity. Nucleotides for which there is no SHAPE data can either be left out of the file, or the reactivity can be entered as less than -500. Columns are separated by any white space.

MC-Sym
^^^^^^^^^^^^^

FAQ
~~~~~~~~~~~~~

- Does it work for more than one chain??? Hmm.. I think it's not. You have to check on your own. --magnus

TIPS
~~~~~~~~~~~~~

Should you need to run it on a list of sequences, use the following script::

    from rna_tools import Seq
    f = open("listOfSequences.fasta")
    for line in f:
     if line.startswith('>'):
       print line,
     else:
       print line,
       s = Seq.Seq(line.strip()) # module first Seq and class second Seq #without strip this has two lines
       print s.predict_ss(method="contextfold"),
       #print s.predict_ss(method="centroid_fold")

TODO
~~~~~~~~~~~~~

- This calss should be renamed to RNASeq and merged with RNASeq class from RNAalignment

"""  # noqa
import os

try:
    RPT_PATH = os.environ['RNA_TOOLS_PATH']
except KeyError:
    print ('Set up RNA_TOOLS_PATH, see Installation note')

import subprocess
import tempfile
import sys

from rna_tools.SecondaryStructure import parse_vienna_to_pairs
from rna_tools.rna_tools_config import CONTEXTFOLD_PATH, RNASTRUCTURE_PATH, ENTRNA_PATH


class MethodNotChosen(Exception):
    pass


class RNASequence(object):
    """RNASequence.

    Usage::

        >>> seq = RNASequence("CCCCUUUUGGGG")
        >>> seq.name = 'RNA03'
        >>> print(seq.predict_ss("RNAfold", constraints="((((....))))"))
        >RNA03
        CCCCUUUUGGGG
        ((((....)))) ( -6.40)

    """

    def __init__(self, seq, ss='', name='rna_seq'):
        self.seq = seq
        self.ss = ss
        self.ss_log = ''
        self.name = name

    def __repr__(self):
        return self.name + '\n' + self.seq + '\n' + self.ss

    def eval(self, ss='', no_dangling_end_energies=False, verbose=False):
        """Evaluate energy of RNA sequence.

        Args:
            ss (optional), if not set, then self.ss is taken for calc
            no_dangling_end_energies (Boolean)
            verbose (Boolean)

        Returns:
            Energy (float)

        The RNAeval web server calculates the energy of a RNA sequence on a given secondary structure.
        You can use it to get a detailed thermodynamic description (loop free-energy decomposition) of your RNA structures.

        Simply paste or upload your sequence below and click Proceed. To get more information on the meaning of the options click the help symbols. You can test the server using this sample sequence/structure pair.

        An equivalent RNAeval command line call would have been::

            RNAeval -v -d0 < input.txt

        Read more: <http://rna.tbi.univie.ac.at//cgi-bin/RNAWebSuite/RNAeval.cgi>

        """
        tf = tempfile.NamedTemporaryFile(delete=False)
        if not ss:
            ss = self.ss

        tf.name += '.fa'
        with open(tf.name, 'w') as f:
            f.write('>' + self.name + '\n')
            f.write(self.seq + '\n')
            f.write(ss + '\n')

        dopt = ' -d2 '
        if no_dangling_end_energies:
            dopt = ' -d0 '

        cmd = 'RNAeval ' + dopt + ' < ' + tf.name
        if verbose:
            print(cmd)
        self.ss_log = subprocess.check_output(cmd, shell=True).decode()
        # [u'>rna_seq\nGGCAGGGGCGCUUCGGCCCCCUAUGCC\n((((((((.((....)).)))).))))', u'(-13.50)']
        return float(self.ss_log.strip().split(' ')[-1].replace('(','').replace(')', ''))


    def get_foldability(self, ss='', verbose=False):
        """Calculate foldability based on EntRNA.

        Steps:

        - parse SS into basepairs,
        - calculate foldabilty

        Configuration:

        - Set ENTRNA_PATH to the folder where ENTRNA_predict.py is.

        Cmd wrapper in here::

            python ENTRNA_predict.py --seq_file pseudoknotted_seq.txt --str_file pseudoknotted_str.txt

        Su, C., Weir, J. D., Zhang, F., Yan, H., & Wu, T. (2019).
        ENTRNA: a framework to predict RNA foldability. BMC Bioinformatics, 20(1), 1–11.
        http://doi.org/10.1186/s12859-019-2948-5
        """
        if ss:
            self.ss = ss

        # parse SS into base-pairs
        def dp_to_bp(dp):
            import numpy as np
            a_list = []
            bp_array = np.zeros(len(dp),dtype = int)
            for i in range(len(dp)):
                if dp[i] == "(":
                    a_list.append(i)
                if dp[i] == ")":
                    bp_array[i] = a_list[-1] + 1
                    bp_array[a_list[-1]] = i + 1
                    a_list.pop()
            return list(bp_array)

        bp = dp_to_bp(self.ss)
        if verbose: print(bp)

        tempstr = tempfile.NamedTemporaryFile(delete=False)
        with open(tempstr.name, 'w') as f:
            f.write(str(bp))

        tempseq = tempfile.NamedTemporaryFile(delete=False)
        with open(tempseq.name, 'w') as f:
            f.write(self.seq)

        # -W to silent warnings See [1]
        cmd = "cd " + ENTRNA_PATH + " && python -W ignore ENTRNA_predict.py --seq_file " + tempseq.name + " --str_file " + tempstr.name
        log = subprocess.check_output(cmd, shell=True).decode()
        if verbose:
            print(cmd)
            print(log)
        for l in log.split('\n'):
            if l.startswith('Foldability: '):
                return round(float(l.replace('Foldability: ', '')), 2)
        return -1
        ## [1]:
        ##     /Users/magnus/work/evoClustRNA/rna-foldability/ENTRNA/util/pseudoknot_free.py:22: SettingWithCopyWarning:
        ## A value is trying to be set on a copy of a slice from a DataFrame.
        ## Try using .loc[row_indexer,col_indexer] = value instead

        ## See the caveats in the documentation: http://pandas.pydata.org/pandas-docs/stable/indexing.html#indexing-view-versus-copy
        ##   df_v1['length'] = df_v1['seq'].apply(lambda x:len(x))
        ## /home/magnus/miniconda2/lib/python2.7/site-packages/sklearn/linear_model/logistic.py:433: FutureWarning: Default solver will be changed to 'lbfgs' in 0.22. Specify a solver to silence this warning.
        ##   FutureWarning)
        ## cd /Users/magnus/work/evoClustRNA/rna-foldability/ENTRNA/ && python ENTRNA_predict.py --seq_file /var/folders/yc/ssr9692s5fzf7k165grnhpk80000gp/T/tmpUORegp --str_file /var/folders/yc/ssr9692s5fzf7k165grnhpk80000gp/T/tmp1ERCcD

    def predict_ss(self, method="RNAfold", constraints='', enforce_constraint=False, shapefn='', explore='', verbose=0):
        """Predict secondary structure of the seq.

        Args:
          method:
          onstraints:
          shapefn (str): path to a file with shape reactivites
          verbose (boolean)

        It creates a seq fasta file and runs various methods for secondary structure
        prediction. You can provide also a constraints file for RNAfold and RNAsubopt.

        Methods that can be used with contraints: RNAsubopt, RNAfold, mcfold.

        Methods that can be used with SHAPE contraints: RNAfold.

        **ContextFold**

        Example::

            $ java -cp bin contextFold.app.Predict in:CCCCUUUGGGGG
            CCCCUUUGGGGG
            ((((....))))

        It seems that a seq has to be longer than 9. Otherwise::

            $ java -cp bin contextFold.app.Predict in:UUUUUUGGG
            Exception in thread "main" java.lang.ArrayIndexOutOfBoundsException: 10

            # this is OK
            $ java -cp bin contextFold.app.Predict in:CCCCUUUGGG
            CCCCUUUGGG
            .(((...)))


        **RNAstructure**

        Example::

            >>> seq = RNASequence("GGGGUUUUCCC")
            >>> print(seq.predict_ss("rnastructure"))
            >  ENERGY = -4.4  rna_seq
            GGGGUUUUCCC
            ((((...))))

        and with the shape data::

            >>> print(seq.predict_ss("rnastructure", shapefn="data/shape.txt"))
            >  ENERGY = -0.2  rna_seq
            GGGGUUUUCCC
            .(((....)))

        the shape data::

            1 10
            2 1
            3 1

        You can easily see that the first G is unpaired right now! The reactivity of this G was
        set to 10. Worked!


        **MC-Fold**

        MC-Fold uses the online version of the tool, this is very powerful with constraints::

            rna_seq
            acucggcuaggcgaguauaaauagccgucaggccuagcgcguccaagccuagccccuucuggggcugggcgaagggucggg
            ((((........)))).......((((..............(((((((((((((((....)))))))))))))))..))))
            curl -Y 0 -y 300 -F "pass=lucy" -F mask="((((........)))).......((((..............(((((((((((((((....)))))))))))))))..))))" -F sequence="acucggcuaggcgaguauaaauagccgucaggccuagcgcguccaagccuagccccuucuggggcugggcgaagggucggg" https://www.major.iric.ca/cgi-bin/MC-Fold/mcfold.static.cgi
            mcfold::energy best dynamics programming: -53.91
            (-53.91, '((((........)))).......((((..............(((((((((((((((....)))))))))))))))..))))')

            curl -Y 0 -y 300 -F "pass=lucy" -F mask="((((........)))).......((((..............((((((((((..............))))))))))..))))" -F sequence="acucggcuaggcgaguauaaauagccgucaggccuagcgcguccaagccuagccccuucuggggcugggcgaagggucggg" https://www.major.iric.ca/cgi-bin/MC-Fold/mcfold.static.cgi
            mcfold::energy best dynamics programming: -34.77
            (-34.77, '((((........)))).......((((..............((((((((((..............))))))))))..))))')

            acucggcuaggcgaguauaaauagccgucaggccuagcgcguccaagccuagccccuucuggggcugggcgaagggucggg
            ((((........)))).......((((..............(((((((((((((((....)))))))))))))))..))))
            curl -Y 0 -y 300 -F "pass=lucy" -F mask="((((xxxxxxxx))))xxxxxxx((((xxxxxxxxxxxxxx((((((((((xxxxxxxxxxxxxx))))))))))xx))))" -F sequence="acucggcuaggcgaguauaaauagccgucaggccuagcgcguccaagccuagccccuucuggggcugggcgaagggucggg" https://www.major.iric.ca/cgi-bin/MC-Fold/mcfold.static.cgi
            mcfold::energy best dynamics programming: -34.77
            (-34.77, '((((........)))).......((((..............((((((((((..............))))))))))..))))')


            acucggcuaggcgaguauaaauagccgucaggccuagcgcguccaagccuagccccuucuggggcugggcgaagggucggg
            ((((........)))).......((((..............(((((((((((((((....)))))))))))))))..))))
            curl -Y 0 -y 300 -F "pass=lucy" -F mask="((((********))))*******((((**************((((((((((**************))))))))))**))))" -F sequence="acucggcuaggcgaguauaaauagccgucaggccuagcgcguccaagccuagccccuucuggggcugggcgaagggucggg" https://www.major.iric.ca/cgi-bin/MC-Fold/mcfold.static.cgi
            mcfold::energy best dynamics programming: -77.30
            (-71.12, '(((((((..))))))).......((((((.(((...)))..(((((((((((((((....)))))))))))))))))))))')

            acucggcuaggcgaguauaaauagccgucaggccuagcgcguccaagccuagccccuucuggggcugggcgaagggucggg
            ((((........)))).......((((..............(((((((((((((((....)))))))))))))))..))))
            curl -Y 0 -y 300 -F "pass=lucy" -F mask="((((**[[[[[**))))*******((((****]]]]]****(((((((((((((((****)))))))))))))))**))))" -F sequence="acucggcuaggcgaguauaaauagccgucaggccuagcgcguccaagccuagccccuucuggggcugggcgaagggucggg" https://www.major.iric.ca/cgi-bin/MC-Fold/mcfold.static.cgi
            mcfold::energy best dynamics programming: -77.30
            ('-77.30', '((((**[[[[[**))))*******((((****]]]]]****(((((((((((((((****)))))))))))))))**))))')

        **explore**

         The sub-optimal search space can be constrained within a percentage of the minimum free energy structure, as MC-fold makes use of the Waterman-Byers algorithm [18, 19]. Because the exploration has an exponential time complexity, increasing this value can have a dramatic effect on MC-Fold’s run time.

        Parisien, M., & Major, F. (2009). RNA Modeling Using the MC-Fold and MC-Sym Pipeline [Manual] (pp. 1–84).


        """

        tf = tempfile.NamedTemporaryFile(delete=False)
        tf.name += '.fa'
        with open(tf.name, 'w') as f:
            f.write('>' + self.name + '\n')
            f.write(self.seq + '\n')
            if constraints:
                f.write(constraints)

        # check for seq and constraints
        if constraints:
            if len(self.seq) != len(constraints):
                raise Exception('The seq and constraints should be of the same length: %i %s %i %s' % (len(self.seq), self.seq, len(constraints), constraints))

        # run prediction
        # rnafold without contraints
        if method == "RNAfold" and constraints:
            cmd = 'RNAfold -C < ' + tf.name
            if verbose:
                print(cmd)
            self.ss_log = subprocess.check_output(cmd, shell=True).decode()
            return '\n'.join(self.ss_log.strip().split('\n')[:])

        if method == "RNAfoldX" and constraints:
            if enforce_constraint:
                cmd = 'RNAfold -p -d2 --noLP -C --enforceConstraint < ' + tf.name
            else:
                cmd = 'RNAfold -p -d2 --noLP -C < ' + tf.name
            if verbose:
                print(cmd)

            try:
                self.ss_log = subprocess.check_output(cmd, shell=True).decode()
            except subprocess.CalledProcessError:
                 print('Error')
                 return 0, 'error', 0, '', 0, '', 0, 0

            if verbose:
                print(self.ss_log)
            # parse the results
            lines = self.ss_log.split('\n')
            if 'Supplied structure constraints create empty solution set for sequence' in self.ss_log:
                return 0, 'Supplied structure constraints create empty solution set for sequence', 0, '', 0, '', 0, 0
            #if not 'frequency of mfe structure' in self.ss_log:

            # RNAfold -p -d2 --noLP -C < /var/folders/yc/ssr9692s5fzf7k165grnhpk80000gp/T/tmpGiUoo7.fa
            # >rna_seq
            # AAAUUAAGGGGAAGCGUUGAGCCGCUACCCAUAUGUGGUUCACUCGGAUAGCGGGGAGCUAAUAGUGAAACCGGCCCUUUAGGGG
            # ...((((((((.(((......((((((.((....(((...)))..)).))))))...)))..............))))))))... (-19.80)
            # ...{(((((((.(((......((((((.((....(((...)))..)).))))))...)))..............)))))))}... [-21.05]
            #...((((((((.(((......((((((.((....(((...)))..)).))))))...)))..............))))))))... {-19.80 d=2.34}
            # frequency of mfe structure in ensemble 0.131644; ensemble diversity 3.68

            mfess = lines[2].split()[0]
            mfe = ' '.join(lines[2].split()[-1:])
            mfe = float(mfe.replace('(', '').replace(')', '')) # (-19.80) ->-19.80

            efess = lines[3].split()[0]  # ensamble free energy
            efe = ' '.join(lines[3].split()[-1:])
            efe = float(efe.replace('[', '').replace(']', '')) # (-19.80) ->-19.80

            cfess = lines[4].split()[0]  # ensamble free energy
            cfe, d = ' '.join(lines[4].split()[1:]).split('d')
            cfe = float(cfe.replace('{', '').replace('}', '')) # (-19.80) ->-19.80

            words = lines[5].split()  # ensamble free energy
            freq = round(float(words[6].replace(';', '')), 2) # frequency of mfe structure in ensemble
            diversity = float(words[9])     # ensemble diversity

            if verbose:
                print(mfe, mfess, efe, efess, cfe, cfess, freq, diversity)
            return mfe, mfess, efe, efess, cfe, cfess, freq, diversity

        elif method == "RNAfold":
            cmd = 'RNAfold < ' + tf.name
            if verbose:
                print(cmd)
            self.ss_log = subprocess.check_output(cmd, shell=True).decode()
            return '\n'.join(self.ss_log.strip().split('\n')[:])

        elif method == "RNAsubopt" and constraints:
            cmd = 'RNAsubopt -C < ' + tf.name
            if verbose:
                print(cmd)
            self.ss_log = subprocess.check_output(cmd, shell=True).decode()
            return '\n'.join(self.ss_log.split('\n')[:])

        elif method == "RNAsubopt":
            cmd = 'RNAsubopt < ' + tf.name
            if verbose:
                print(cmd)
            self.ss_log = subprocess.check_output(cmd, shell=True).decode()
            return '\n'.join(self.ss_log.split('\n')[:])

        elif method == "mcfold":
            #  -F tope=1
            if explore:
                explore_str = " -F explore=" + str(explore)
            else:
                explore_str = ''

            #if constraints:
            #cmd = "curl -Y 0 -y 300 -F \"pass=lucy\" -F mask=\"" + constraints + "\" " + explore_str + \
            #" -F sequence=\"" + self.seq + "\" https://www.major.iric.ca/cgi-bin/MC-Fold/mcfold.static.cgi"
            cmd = "curl https://www.major.iric.ca/cgi-bin/MC-Fold/mcfold.static.cgi\?pass\=lucy\&sequence\=" + self.seq + "\&top\=20\&explore\=15\&name\=\&mask\='" + constraints + "'\&singlehigh\=\&singlemed\=\&singlelow\="
            # cmd = "curl -Y 0 -y 300 -F \"pass=lucy\" -F sequence=\"" + self.seq + "\" https://www.major.iric.ca/cgi-bin/MC-Fold/mcfold.static.cgi"
            if verbose:
                print(cmd)
                
            o = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            out = o.stdout.read().decode(errors='ignore').strip()
            err = o.stderr.read().decode(errors='ignore').strip()

            if verbose:
                print(out)

            # If the structure can't be find, detect this statement and finish this routine.
            if 'Explored 0 structures' in out:
                return 0.00, '', 'Explored 0 structures'

            comment = ''
            energy = ''
            out = out.split('\n')
            for l in out :
                # first you will find the best dynamic energy, and in the next loop
                # it will be used to search for lines with this energy and secondary
                # structure

                # (((..)))  -5.43
                if energy:  # if energy is set
                    if energy in l:
                        if verbose: print(l)
                        ss = l.split()[0]

                # Performing Dynamic Programming...
                # Best Dynamic Programming Solution has Energy:  -5.43
                if l.startswith('Best Dynamic Programming Solution has Energy:'):
                    energy_bdp = l.split(':')[1].strip()
                    if verbose:
                        print ('mcfold::energy best dynamics programming: ' + energy_bdp)
                        comment = 'energy best dynamics programming'
                    ss = constraints
                    # return float(energy), constraints # I'm not sure if this is good

            # Ok, for whatever reason Best DP energy might not be exactly the same as and
            # the energy listed later for secondary structure. So this code finds this secondary
            # structure and gets again the energy for this secondary structure,
            # and overwrites the previous energy.
            # In this case:
            # Best Dynamic Programming Solution has Energy:  -5.46
            # ...
            # CUCUCGAAAGAUG
            # (((.((..)))))  -5.44 ( +0.00)
            # (((.((..)))))  BP >=  50%

            # if evenn this will not find ss, then set ss to null not to crash
            # and it's possible, like in here
            # curl -Y 0 -y 300 -F "pass=lucy" -F mask="((******)))" -F sequence="CCUgcgcaAGG" \
            #            http://www.major.iric.ca/cgi-bin/MC-Fold/mcfold.static.cgi
            ss = ''
            for l in out:
                if 'target="_blank">MARNA</a>-formatted:<P><P><P></H2><pre>' in l:
                    index = out.index(l)
                    ss_line = out[index + 2]
                    ss, energy = ss_line.split()[0:2]  # '(((.((..)))))  -5.44 ( +0.00)'

                    # if there is
                    # UUGCCGUAAGACA
                    # .............  BP >=  50%
                    # then finish with energy 0.00, and empty ss
                    if energy == 'BP':
                        energy = energy_bdp
                        comment = 'BP energy'
                        return energy_bdp, constraints, comment
                    # break

            # prepare outputs, return and self-s
            self.log = out
            self.ss = ss
            return float(energy), ss, comment

        # if method == "RNAsubopt":
        #     from cogent.app.vienna_package import RNAfold, RNAsubopt
        #     r = RNAsubopt(WorkingDir="/tmp")
        #     res = r([self.seq])
        #     return str(res['StdOut'].read()).strip()

        # if method == 'RNAfold':
        #     from cogent.app.vienna_package import RNAfold, RNAsubopt
        #     r = RNAfold(WorkingDir="/tmp")
        #     res = r([self.seq])
        #     self.ss_log = res['StdOut'].read()
        #     return self.ss_log.strip().split('\n')[-1].split()[0]

        elif method == "ipknot":
            self.ss_log = subprocess.check_output('ipknot ' + tf.name, shell=True)
            return '\n'.join(self.ss_log.split('\n')[2:])

        elif method == "contextfold":
            if not CONTEXTFOLD_PATH:
                print('Set up CONTEXTFOLD_PATH in configuration.')
                sys.exit(0)
            cmd = "cd " + CONTEXTFOLD_PATH + \
                  " + && java -cp bin contextFold.app.Predict in:" + self.seq
            if verbose:
                print(cmd)
            self.ss_log = subprocess.check_output(cmd, shell=True).decode()
            return '\n'.join(self.ss_log.split('\n')[1:])

        elif method == "centroid_fold":
            self.ss_log = subprocess.check_output('centroid_fold ' + tf.name, shell=True)
            return '\n'.join(self.ss_log.split('\n')[2:])

        elif method == "rnastructure":
            cmd = RNASTRUCTURE_PATH + '/exe/Fold ' + tf.name + ' ' + tf.name + '.out '
            if shapefn:
                cmd += ' -sh ' + shapefn
            if verbose:
                print(cmd)
            o = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            stderr = o.stderr.read().strip()
            if stderr:
                print(stderr)

            cmd = RNASTRUCTURE_PATH + '/exe/ct2dot ' + tf.name + '.out 1 ' + \
                             tf.name + '.dot'
            if verbose:
                print(cmd)
            o = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            stderr = o.stderr.read().strip()
            if not stderr:
                with open(tf.name + '.dot') as f:
                    return f.read().strip()

        # (-51.15, '.(.(((((((((((((((..))))))))))))))))(..((((((((....)))).))))).')
        elif method == "rnastructure_CycleFold":
            cmd = RNASTRUCTURE_PATH + '/exe/CycleFold ' + tf.name + ' > ' + tf.name + '.ct '
            if verbose:
                print(cmd)
            o = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            stderr = o.stderr.read().strip()
            if stderr:
                print(stderr)

            # get energy
            energy = float(open(tf.name + '.ct').readline().split("energy:")[1].strip())  # >rna_seq	energy: -51.1500

            # get ss in dot-bracket notation
            cmd = RNASTRUCTURE_PATH + '/exe/ct2dot ' + tf.name + '.ct 1 ' + \
                             tf.name + '.dot'
            if verbose:
                print(cmd)
            o = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            stderr = o.stderr.read().strip()
            if not stderr:
                with open(tf.name + '.dot') as f:
                    # (-51.15, '.(.(((((((((((((((..))))))))))))))))(..((((((((....)))).))))).')
                    return energy, f.read().strip().split('\n')[2]


        else:
            raise MethodNotChosen('You have to define a correct method to use.')


# main

def load_fasta_ss_into_RNAseqs(fn, debug=True):
    seqs = []
    with open(fn) as f:
        for line in f:
            if debug: print(line)
            name = line.replace('>', '').strip()
            seq = next(f).strip()
            ss = next(f).strip()
            rs = RNASequence(seq, ss, name)
            seqs.append(rs)
    return seqs

if __name__ == '__main__':
    import doctest
    doctest.testmod()

    seq = RNASequence("CGCUUCAUAUAAUCCUAAUGAUAUGGUUUGGGAGUUUCUACCAAGAGCCUUAAACUCUUGAUUAUGAAGUG")
    seq.name = 'RNA01'
    print(seq.predict_ss("RNAfold",
                          constraints="((((...............................................................))))"))  # noqa
    seq = RNASequence("CGCUUCAUAUAAUCCUAAUGAUAUGGUUUGGGAGUUUCUACCAAGAGCCUUAAACUCUUGAUUAUGAAGUG")
    seq.name = 'RNA02'
    print(seq.predict_ss("RNAsubopt",
                         constraints="((((...............................................................))))"))  # noqa

    print(seq.predict_ss("contextfold"))
    # print seq.predict_ss(method="ipknot")

    verbose = False
    seq = RNASequence("GGGGUUUUCCC")
    print(seq.predict_ss("rnastructure", verbose=verbose))
    print(seq.predict_ss("rnastructure", shapefn="data/shape.txt", verbose=verbose))

    seq = RNASequence("CGUGGUUAGGGCCACGUUAAAUAGUUGCUUAAGCCCUAAGCGUUGAUAAAUAUCAGgUGCAA")
    print(seq.predict_ss("rnastructure", shapefn="data/shape.txt", verbose=verbose))

    #
    # test of MethodNotChose
    # print(seq.predict_ss("test"))



#!/usr/bin/env python

"""quick and dirty test"""

import unittest
import subprocess
import os

class MyTest(unittest.TestCase):
    """RNA Alignment test"""
    def test_rna_align_find_seq_in_alignment(self):
        print('----------------------------------------------------------------------')
        cmd = "./rna_align_find_seq_in_alignment.py -a test_data/RF00167.stockholm.sto -f test_data/xx.fa"
        print(cmd)
        code = os.system(cmd)
        self.assertEqual(code, 0)
        
    def test_rna_align_find_core(self):
        print('----------------------------------------------------------------------')
        cmd = "./rna_align_find_core.py test_data/RF00167.stockholm.sto"
        print(cmd)
        code = os.system(cmd)
        self.assertEqual(code, 0)

    def test_rna_align_seq_to_alignment(self):
        print('----------------------------------------------------------------------')
        cmd = "./rna_align_seq_to_alignment.py -f test_data/4lvv_cmalign.txt -a test_data/RF01831.stockholm.sto"
        print(cmd)
        code = os.system(cmd)
        self.assertEqual(code, 0)

    def test_rna_align_seq_to_alignment_2(self):
        print('----------------------------------------------------------------------')
        cmd = "./rna_align_seq_to_alignment.py -s test_data/4lvv.seq -a test_data/RF01831.stockholm.sto -m test_data/RF01831.cm"
        print(cmd)
        code = os.system(cmd)
        self.assertEqual(code, 0)

    def test_rna_align_get_ss_from_alignment(self):
        print('----------------------------------------------------------------------')
        cmd = "./rna_align_get_ss_from_alignment.py test_data/ade.fa"
        print(cmd)
        code = os.system(cmd)
        self.assertEqual(code, 0)

    def test_rna_alignment(self):
        print('----------------------------------------------------------------------')
        cmd = "./rna_alignment.py"
        print(cmd)
        code = os.system(cmd)
        self.assertEqual(code, 0)
    def test_random_assignment_of_nucleotides(self):
        print('----------------------------------------------------------------------')
        cmd = "python random_assignment_of_nucleotides.py --alignfn test_data/aln1.fasta"
        print(cmd)
        code = os.system(cmd)
        self.assertEqual(code, 0)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""pefx.py - per folder eXectue cmd.

Usage::

    pefx.py 'ls | grep '.pdb\$' | wc -l'

"""
from __future__ import print_function

import argparse
import os
import glob
import sys
import re


def get_parser():
    parser = argparse.ArgumentParser(
        description=__doc__, formatter_class=argparse.RawDescriptionHelpFormatter)

    parser.add_argument('-d', "--dryrun",
                        action="store_true", help="dry run", default=False)

    parser.add_argument('-p', '--path', help="", default='')
    parser.add_argument('-f', '--folder-only', help="",  action="store_true")
    parser.add_argument('-c', '--case', help="only one case, for test")
    parser.add_argument("-v", "--verbose",
                        action="store_true", help="be verbose")
    parser.add_argument('cmd')
    return parser


def sort_nicely(l):
   """ Sort the given list in the way that humans expect.
   http://blog.codinghorror.com/sorting-for-humans-natural-sort-order/
   """
   convert = lambda text: int(text) if text.isdigit() else text
   alphanum_key = lambda key: [ convert(c) for c in re.split('([0-9]+)', key) ]
   l.sort( key=alphanum_key )
   return l


def exe(cmd, dryrun, verbose):
    if verbose: print(cmd)
    if not dryrun: os.system(cmd)


def main(dryrun, path, case, cmd, folder_only):
    if path:
        os.chdir(path)
    root = os.getcwd()
    cases = sort_nicely(glob.glob('*'))

    for c in cases:
        if folder_only:
            if not os.path.isdir(c):
                continue
        #print('Case: ', c)
        #print(c, end='', flush=True)
        sys.stdout.write(c)
        sys.stdout.flush()
        # mode only for a specific case
        if case: # only if this is used
            if c != case:
                print('!!! skip ' + c + '!!!')
                continue

        # if not break ed, use this
        if os.path.isdir(c):
            # print ('Inside %s' % c)
            os.chdir(c) # go inside a folder
            # cmd
            #subcases = glob.glob('*.fa')
            #for sc in subcases:
            exe(cmd, dryrun, verbose=False)
            os.chdir(root)


if __name__ == '__main__':
    parser = get_parser()
    args = parser.parse_args()

    main(args.dryrun, args.path, args.case, args.cmd, args.folder_only)



#!/usr/bin/env python

"""
cmaling::

    [mm] thf cmalign RF01831.cm 4lvv.seq
    # STOCKHOLM 1.0
    #=GF AU Infernal 1.1.2

    4lvv         -GGAGAGUA-GAUGAUUCGCGUUAAGUGUGUGUGA-AUGGGAUGUCG-UCACACAACGAAGC---GAGA---GCGCGGUGAAUCAUU-GCAUCCGCUCCA
    #=GR 4lvv PP .********.******************9999998.***********.8999999******8...5555...8**************.************
    #=GC SS_cons (((((----(((((((((((,,,,,<<-<<<<<<<<___________>>>>>>>>>>,,,<<<<______>>>>,,,)))))))))))-------)))))
    #=GC RF      ggcaGAGUAGggugccgugcGUuAAGUGccggcgggAcGGGgaGUUGcccgccggACGAAgggcaaaauugcccGCGguacggcaccCGCAUcCgCugcc
    //

Reads seq files::
   
   >4lvv
   GGAGAGUAGAUGAUUCGCGUUAAGUGUGUGUGAAUGGGAUGUCGUCACACAACGAAGCGAGAGCGCGGUGAAUCAUUGCAUCCGCUCCA
"""
import rna_tools.utils.rna_alignment.rna_alignment as ra

import sys
import argparse

def get_parser():
    parser = argparse.ArgumentParser(description=__doc__, formatter_class=argparse.RawDescriptionHelpFormatter)
    parser.add_argument('-f', '--file', help="cmalign output")
    parser.add_argument('-a', '--alignment', help="alignment file",  required=True)
    parser.add_argument('-m', '--cm', help="cm model to run cmalign on")
    parser.add_argument('-s', '--seq', help="seq fn, fasta!")
    return parser

if __name__ == '__main__':
    args = get_parser().parse_args()

    # don't run cmalign
    if args.file:
        cma = ra.CMAlign(outputfn=args.file)
    else:
        cma = ra.CMAlign()#(outputfn=args.file)
        cma.run_cmalign(args.seq, args.cm)

    seq = cma.get_seq()

    a = ra.RNAalignment(args.alignment)
    print('cma hit  ' + seq)
    print('seq      ' + a.align_seq(seq))
    print('a.rf     ' + a.rf)






#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""SimRNATrajectory module.

SimRNATrajectory / Frame / Residue / Atom
"""

from __future__ import print_function
from collections import deque
import numpy as np
import gc


class SimRNATrajectory:
    """SimRNATrajectory"""

    def __init__(self):
        """Crate SimRNATrajectory object, empty.
        Use
        - load_from_file
        - load_from_string
        to load the data.
        """
        self.frames = []

    def load_from_file(self, fn, debug_break=False, top_level=False, only_first_frame=False):
        """Create a trajectory based on give filename.

        Args:

           top_level:
                top_level = True, don't make a huge tree of objects (Residues/Atoms) == amazing speed up!
                Useful if you need only frames, energies and coords as text.
                You only get the info that is in header of each frame.

                top_level = False, makes huge tree of objects (Residues/Atoms) == very slow for a huge trajectories

        .. warning:: Loads up whole trafl file into memory, and get stuck. Use this if you want to compute e.g. distances between atoms, get the positions of specified atoms etc. If you can not process your trajectory
        use top_level=True or look at load_from_string() to load a frame by frame from a file.

        h(eader), l(line), f(ile)
        """
        self.frames = []
        f = (line for line in open(fn))
        h = next(f).strip()
        l = next(f).strip()
        c = 0
        self.frames.append(Frame(c, h, l, top_level))

        if not only_first_frame:
            while 1:
                c += 1
                try:
                    h = next(f).strip()
                except StopIteration:
                    break
                l = next(f).strip()
                if h and l:
                    self.frames.append(Frame(c, h, l, top_level))
                    if debug_break:
                        break
                    if c % 1000 == 0:
                        print(c / 1000, 'k loaded...')
                        gc.collect()

    def load_from_string(self, c, txt):
        """Create a trajectory based on given string (txt) with id given by c.

        Faster method, loads only one frame at a time to memory, and after computations
        loads the next frame (memory efficient)."""
        h, l = txt.split('\n')
        self.frames.append(Frame(c, h, l))

    def sort(self, inplace=True):
        """Sort frames within the trajectory according to energy."""
        def getEnergy(frame):
            return frame.energy
        frames_sorted = sorted(self.frames, key=getEnergy)
        if inplace:
            self.frames = frames_sorted
        else:
            return frames_sorted

    def load_from_list(self, frames):
        self.frames = frames

    def save(self, fn, verbose=True):
        """Save the trajectory to file."""
        with open(fn, 'w') as fi:
            for f in self.frames:
                # [295.0, 5.0, -1428.683789, -1435.583554, 0.9]
                fi.write(' '.join([str(x) for x in f.header]) + '\n')
                fi.write(f.coords + '\n')
        if verbose:
            print('Saved to ' + fn)

    def plot_energy(self, plotfn='plot.png'):
        """Plots the SimRNA energy of the trajectory.

        .. image:: ../pngs/simrnatrajectory.png
        """

        # plotting inside ipython
        import matplotlib.pyplot as plt
        import matplotlib
        plt.plot([f.energy for f in self.frames])
        plt.ylabel('# frames')
        plt.ylabel('energies')
        plt.title('SimRNATrajectory: energies over frames')
        plt.grid()
        plt.savefig(plotfn, figsize=(30, 10))  # bbox_inches='tight',
        plt.close()

    def __len__(self):
        """Get number of frames"""
        return len(self.frames)

    def __getitem__(self, i):
        return self.frames[i]


class Frame:
    """Frame

    Syntax of header:
     - write_number
     - replica_id
     - total_energy
     - energy_without_restraints
     - temperature

    .. warning:: If there is an invalid frame, please use `repair_trafl.py` to fix the trajectory first.
    """

    def __init__(self, id, header, coords, top_level=False):
        """
        top_level = False, makes huge tree of objects (Residues/Atoms) == slower
        Important when you want to compute e.g. disdances between atoms, get the positions of specified atoms etc.
        """
        self.id = id
        self.header = [float(h) for h in header.split()]
        # 443 10 -1211.659921 -1218.294753 1.100000
        l = header.split()
        if len(l) != 5:
            raise Exception('Invalid frame, please use `repair_trafl.py` to fix it.')
        self.energy = float(l[3])

        self.coords = coords
        coords = deque(coords.split())
        if top_level:
            self.residues = []
        else:
            self.residues = []
            c = 1
            while coords:
                p = Atom('p', coords.popleft(), coords.popleft(), coords.popleft())
                c4p = Atom('c4p', coords.popleft(), coords.popleft(), coords.popleft())
                n1n9 = Atom('n1n9', coords.popleft(), coords.popleft(), coords.popleft())
                b1 = Atom('b1', coords.popleft(), coords.popleft(), coords.popleft())
                b2 = Atom('b2', coords.popleft(), coords.popleft(), coords.popleft())
                r = Residue(c, p, c4p, n1n9, b1, b2)
                self.residues.append(r)
                c += 1

    def __repr__(self):
        return 'Frame #' + str(self.id) + ' e:' + str(round(self.energy, 2))

    def __len__(self):
        """Get a number of residues"""
        return len(self.residues)


class Residue:
    """Create Residue object.

       Each residue in SimRNA coarse-grained represantation consists only 5 coarse-grained atoms:

       - backbone: p = phospate group, c4p = sugar moiety
       - nucleotide: n1n9 = N1 for pyrimidines, N9 for purines, b1 = C2 for purines and pyrimidines, \
       b2 = C4 for pyrimidines, C6 for purines
    """

    def __init__(self, id, p, c4p, n1n9, b1, b2):
        self.id = id
        self.p = p
        self.c4p = c4p
        self.n1n9 = n1n9
        self.b1 = b1
        self.b2 = b2
        self.atoms = [p, c4p, n1n9, b1, b2]
        # self.center

    def __sub__(self, other_residue):
        diff = self.get_center() - other_residue.get_center()
        return np.sqrt(np.dot(diff, diff))

    def get_atoms(self):
        """Return all atoms"""
        return self.atoms

    def get_center(self):
        """Return MB for residue ```((self.n1n9 + self.b2) / 2)```"""
        return (self.n1n9 + self.b2) / 2

    def __repr__(self):
        return 'Residue ' + str(self.id)


class Atom:
    """Atom
    x
    y
    z
    coord
    """

    def __init__(self, name, x, y, z):
        """Create Atom object."""
        self.name = name
        self.x = float(x)
        self.y = float(y)
        self.z = float(z)
        self.coord = np.array([self.x, self.y, self.z])

    def get_coord(self):
        """Return coords (np.array)."""
        return self.coord

    def __repr__(self):
        return 'Atom ' + str(self.x) + ' ' + str(self.y) + ' ' + str(self.z)

    def __sub__(self, other_atom):
        """Calculate distance between two atoms.

        Example::

            distance = atom1 - atom2

        """
        diff = self.coord - other_atom.coord
        return np.sqrt(np.dot(diff, diff))

    def __add__(self, other_atom):
        return self.coord + other_atom.coord


# main
if __name__ == '__main__':
    s = SimRNATrajectory()
    s.load_from_file('test_data/6c2ca958-d3aa-43b2-9f02-c42d07a6f7e9_ALL.trafl', top_level=True)
    s.plot_energy('plot.png')

    s = SimRNATrajectory()
    s.load_from_file(
        'test_data/8b2c1278-ee2f-4ca2-bf4a-114ec7151afc_ALL_thrs6.20A_clust01.trafl', top_level=False)
    for f in s.frames:
        print('header of each frame:', f.header)
        # prints out the energy of restraints (total_energy - energy_without_restraints)
        print(f.header[3] - f.header[4])
        print(f.coords)  # prints coords of each coarse-grained atom

    s2 = SimRNATrajectory()
    traj = """1 1 1252.257530 1252.257530 0.950000
 53.570 23.268 39.971 55.119 24.697 43.283 55.145 27.966 42.270 55.258 29.321 42.618 54.313 29.909 40.572 57.246 41.229 41.492 57.056 39.572 45.104 55.672 36.799 43.722 55.491 33.284 44.069 55.013 33.922 41.769"""

    s2.load_from_string(0, traj)
    for f in s2.frames:
        print(f)

        # print f.coords
        r = f.residues[0]
        print(r.get_atoms())
        print(r.p)  # prints the position of P group

        print(r.c4p)
        print(r.c4p.get_coord())
        print(r.b1 - r.p)  # prints the distance between C4 and P coarse grained-atoms

        print(r.n1n9)
        print(r.b2)
        print(r.get_center())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Usage::

  $ rna_rosetta_n.py ade.out
  21594

"""
from subprocess import call

import subprocess
import argparse

def get_parser():
    parser = argparse.ArgumentParser()
    parser.add_argument('src', help='')
    parser.add_argument('dst', help='')
    return parser

#main
if __name__ == '__main__':
    args = get_parser().parse_args()
    call("cp -rv " + args.src + " " + args.dst, shell=True)
    



#!/usr/bin/python

"""Get the restrains for SimRNA based on a file with interactions.

simrna_get_restraints.py <interactions_mapped.txt>

Warning: it only works for chain A!
"""

import pandas as pd
import sys

def get_restraints(filename):
    restraints=[]
    df=pd.read_csv(filename,sep=" ")
    txt = ''
    for row in df.iterrows():
        i = row[1]["i"]
        j = row[1]["j"]
        txt += 'SLOPE A/' + str(i) + '/MB A/' + str(j) + '/MB 3 8 0.5 \n' # energy values about restrains. slope <-> penalty
        txt += 'WELL A/' + str(i) + '/MB A/' + str(j) + '/MB 3 8 0.5 \n' # well <-> reward # check paper
    print txt.strip()

def usage():
    if len(sys.argv)<2:
        print sys.argv[0] + " <interactions_parsed_mapped.csv> "
        print '           output goes to stdout'
        sys.exit()

if __name__=="__main__":
    usage()
    get_restraints(sys.argv[1])



    fn = 'test_data/cat_chunk003_2r8s_5kcycles_decoys_nonativefrags.cluster1.0_clean_noC.pdb'
    pdblines = make_rna_gromacs_ready(open(fn).read())
    print(pdblines)
    with open('test_output/gromacs_ready.pdb', 'w') as f:
        f.write(pdblines)

    fn = 'test_data/cat_chunk003_2r8s_5kcycles_decoys_nonativefrags.cluster1.0_clean_noC.pdb'
    pdblines = make_rna_rnakb_ready(open(fn).read())
    fready = 'test_output/rnakb_ready.pdb'
    print(pdblines)
    with open(fready, 'w') as f:
        f.write(pdblines)

    # prepare groups
    groups_txt, energygrps, seq_uniq = prepare_groups(fready, LIB_PATH + '/rnakb_utils/test_output/groups.txt', potential='5pt', verbose=True)
    print(groups_txt)
    fout = 'test_data/out.mdp'



#!/usr/bin/env python

"""Calculate statistics of foldability on an alignment.

The tool uses ENTRANA [1] to calculate, what the authors called, foldability (column: "foldability") of a given sequence into a given secondary structure.

Next, MC-Fold [2] is executed to calculate free energy (column: "mcsym") on the sequence and the secondary structure obtained based on the alignment. The secondary structure is used as constraints.

The third used program is RNAfold from the Vienna package [3]. Also, in this case the secondary structure obtained with rna-tools from the RNA alignment is used as constraints, columns: "mfe" (minimum free energy), "mfess" (secondary structure for minimum free energy state), "cfe" (minimum free energy of centroid), "cfess" (secondary structure for centroid, "diversity" (ensemble diversity), "efe" (free energy of the thermodynamic ensemble), "efess" (secondary structure for the thermodynamic ensemble), "freq" (frequency of mfe structure in ensemble). RNAfold is also executed in with "--enforceConstraint" where the constraints are enforced. This run gives analogous values as the default RNAfold, to all RNAfold column "_enforce" is added.

The tool is able to calculate the distance Levenshtein (the difference between the two sequences)(column: "distance") from the target sequence and all sequence in the alignment to test if there is a bias in the accuracy towards the most similar sequences.

Another tool used from the Vienna package is RNAeval. The tool calculates free energy for a given sequence and secondary structure.

The accuracy is expressed as the median of core RMSD of 10% the lowest core RMSD models for the given sequences.

.. image :: ../../rna_tools/tools/rna_alignment/test_data/foldability/rp17_foldability.png

The correlations::

    accuracy             1.000000
    cfe                  0.653813
    foldability          0.622038
    mfe                  0.607340
    efe                  0.585077
    diversity            0.404350
    eval                 0.349499
    cfe_enforce          0.311744
    mfe_enforce          0.302973
    efe_enforce          0.280929
    distance             0.256870
    freq                 0.037037
    diversity_enforce    0.018429
    mcsym                0.017533
    freq_enforce        -0.037991
    length              -0.340809

The data:

We tested correlations between the above-mentioned statistics, and the highest correlation, 0.65 () was achieved to the centroid free energy calculated with RNAFold, which suggests that to some extent this metric could be used to pick sequence from the alignment to pick sequences that are more likely to fold.

However, this needs further investigation and the detailed analysis an all cases and more folded sequences.

1. Su C, Weir JD, Zhang F, Yan H, Wu T. ENTRNA: a framework to predict RNA foldability. BMC Bioinformatics. BioMed Central 2019
2. Parisien M, Major F. The MC-Fold and MC-Sym pipeline infers RNA structure from sequence data. Nature 2008;452:51-5
3. Lorenz R, Bernhart SH, Honer Zu Siederdissen C, Tafer H, Flamm C, Stadler PF, et al. ViennaRNA Package 2.0. Algorithms Mol Biol. BioMed Central; 2011;6:26-14.

Example::

    $ python rna_align_foldability.py test_data/gmp_ref.sto test_data/gmp_foldability.csv
                                           cfess_enforce  distance  diversity
    0  ((((((.(((......(((((((((..............)))))))...      1.00       3.96
    1  (((.(((((((..((......(((((.(((((.......))).......      0.69       5.56
    2                                                         0.73       3.84
    3                                                         0.69       5.92
    4  (((....((((.((((((...((((((((...............))...      0.73       7.49
    5  ((((.(((.(((..((..((((((((.................)))...      0.75       7.92
    6  .(((((((((.((((.....(((((((((...............))...      0.72       5.83
    7                                                         0.72       7.35
    8  ...((((((((.(((......((((((.((....(((...)))..)...      0.65       4.86

       diversity_enforce    efe  efe_enforce
    0               2.89 -14.77       -13.75
    1               3.70 -19.52       -18.25
    2               0.00 -15.41         0.00
    3               0.00 -13.55         0.00
    4               2.46  -8.58        -6.91
    5               6.37 -20.72       -20.08
    6               2.92 -11.87       -11.38
    7               0.00 -14.59         0.00
    8               3.83 -21.16       -20.64

                                                   efess
    0  ((((((..........(((((((((..............)))))))...
    1  {{..(((((((..((......(((((.(({((.......}}).......
    2  .......((((.(((..(...(((((((((..............))...
    3  .....((((((.(((.....(((((((((.......{{...,}..,...
    4  .......{(((.,{{{{,...((((((((...............))...
    5  {(((.(((.(((..((..((((((((,{{...,.....)}}..)))...
    6  .(((((((((.((((.....(((((((((...............))...
    7  .....{,.{{{.(((......((((((((((..........,.}))...
    8  ...{(((((((.(((......((((((.((....(((...)))..)...

                             ...                         length   mcsym
    0                        ...                           75.0  -39.73
    1                        ...                           85.0  -37.89
    2                        ...                           84.0  -35.40
    3                        ...                           86.0  -36.11
    4                        ...                           83.0  -37.37
    5                        ...                           80.0  -43.59
    6                        ...                           84.0  -42.95
    7                        ...                           84.0  -36.55
    8                        ...                           85.0  -43.58

                          mcsym comment
    0  energy best dynamics programming
    1                         BP energy
    2                         BP energy
    3                         BP energy
    4  energy best dynamics programming
    5  energy best dynamics programming
    6  energy best dynamics programming
    7                         BP energy
    8  energy best dynamics programming

                                                mcsym ss   mfe mfe_enforce
    0  ((((((.(((......((((((((................))))))... -13.9       -12.9
    1  (((.(((((((..((......(((((.((.................... -18.0       -17.3
    2  ....(..((((.(((......((((((((................)... -14.0         0.0
    3  .(...((((((.(((......((((((((.................... -12.0         0.0
    4  (((....((((.(((......((((((((...............))...  -7.2        -6.1
    5  ((((.(((.(((......((((((((.................)))... -18.6       -18.6
    6  .(((((((((.(((......((((((((.................)... -10.5       -10.5
    7  ...(.((.(((.(((......((((((((................)... -12.8         0.0
    8  ...((((((((.(((......((((((.(.................... -19.8       -19.8

                                                   mfess
    0  ((((((..........(((((((((..............)))))))...
    1  ((..(((((((..((......(((((.(((((.......))).......
    2  .......((((.(((......(((((((((..............))...
    3  .....((((((.(((.....(((((((((....................
    4  .....................((((((((...............))...
    5  ((((.(((.(((..((..(((((((((((.........)))..)))...
    6  .(((((((((.((((.....(((((((((...............))...
    7  .....((.(((.(((......((((((((((............)))...
    8  ...((((((((.(((......((((((.((....(((...)))..)...

                                           mfess_enforce
    0  ((((((.(((......(((((((((..............)))))))...
    1  (((.(((((((..((......(((((.(((((.......))).......
    2                                              error
    3                                              error
    4  (((....((((.((((((...((((((((...............))...
    5  ((((.(((.(((..((..(((((((((((.........)))..)))...
    6  .(((((((((.((((.....(((((((((...............))...
    7                                              error
    8  ...((((((((.(((......((((((.((....(((...)))..)...

                                                     seq
    0  GCGCGGAAACAAUGAUGAAUGGGUUUAAAUUGGGCACUUGACUCAU...
    1  CUGUCGAAGAGACGCGAUGAAUCCCGCCCUGUAAUUCGGGCACCUC...
    2  AAUCAAUAGGGAAGCAACGAAGCAUAGCCUUUAUAUGGACACUUGG...
    3  AAAUAUUAUAGAGAUGUUGAAGUAUAUUCUAUUAUUGGGCACCUUA...
    4  AUUUUAAGAGGAAAUUUUGAACUAUAUACUUAUUUGGGCACUUUGU...
    5  UGCAAUGGGUGUGAUGAAGUCCGGACAGUAAUGUGGGCACUUAGUC...
    6  AAUAUUUUAGAAACUGAGAAGUAUAUCUUAUUAUUGGGCAUCUGGA...
    7  AUAACGGCACGAAGCAAUGAAAUGUUCGAUGUAACCGGGCACCUAU...
    8  AAAUUAAGGGGAAGCGUUGAGCCGCUACCCAUAUGUGGUUCACUCG...

                                                      ss
    0  ((((((.(((......((((((((................))))))...
    1  (((.(((((((..((......(((((.((....................
    2  ....(..((((.(((......((((((((................)...
    3  .(...((((((.(((......((((((((....................
    4  (((....((((.(((......((((((((...............))...
    5  ((((.(((.(((......((((((((.................)))...
    6  .(((((((((.(((......((((((((.................)...
    7  ...(.((.(((.(((......((((((((................)...
    8  ...((((((((.(((......((((((.(....................

    [9 rows x 26 columns]

"""
from __future__ import print_function
from rna_tools.Seq import RNASequence, load_fasta_ss_into_RNAseqs
from rna_tools.tools.rna_alignment.rna_alignment import clean_seq_and_ss, RNAalignment
import argparse
import pandas as pd

def hr(*text):
    print()
    print(' '.join(text))
    print('------------------------------')

def get_parser():
    parser = argparse.ArgumentParser(description=__doc__, formatter_class=argparse.RawDescriptionHelpFormatter)
    parser.add_argument('file', help="an alignment in the Stokholm format, the first seq will be used to calculate distance to (#TODO pick any seq)")
    parser.add_argument('output', help="csv pandas file")
    parser.add_argument("--all-stars", help="this takes usully super long", action="store_true")
    parser.add_argument("--dev",  action="store_true")
    parser.add_argument("--skip-mcfold",  action="store_true")
    parser.add_argument("-v", "--verbose",
                        action="store_true", help="be verbose")
    return parser

if __name__ == '__main__':
    args = get_parser().parse_args()
    a = RNAalignment(args.file)
    target_seq = ''
    df = pd.DataFrame()
    for s in a:
        if 'XX' in s.seq:  # skip line with X
            continue
        if not target_seq:
            target_seq = s.seq

        dist = s.get_distance_to(target_seq)
        s.remove_gaps()
        length = len(s)

        s2 = RNASequence(s.seq, s.ss)
        cst = s.ss
        if args.all_stars:
            cst = s.ss.replace('.', '*')
            explore=1
        else:
            explore=''
        if not args.skip_mcfold:
            ss = list(s2.predict_ss(method='mcfold', constraints=cst, explore=explore, verbose=args.verbose))
            ss[0] = str(ss[0]) # str first value
            if args.verbose: print(ss)
        else:
            ss = '','', ''
        foldability = str(s2.get_foldability())
        eval = str(s2.eval(verbose = args.verbose))
        mcsym = ss[0]

        # RNAfoldX
        mfe, mfess, efe, efess, cfe, cfess, freq, diversity = s2.predict_ss("RNAfoldX", constraints=cst, enforce_constraint=False, verbose=args.verbose)

        mfe_enforce, mfess_enforce, efe_enforce, efess_enforce, cfe_enforce, cfess_enforce, freq_enforce, diversity_enforce = s2.predict_ss("RNAfoldX", constraints=cst, enforce_constraint=True, verbose=args.verbose)
        print('=======================================', s.id)
        print(s.seq)
        print(s.ss)
        print('foldability:', foldability)

        hr('RNAeval:', eval)
        print()
        print('MC-Fold')
        print(ss[1], ss[0], ss[2])

        hr('RNAfold with cst')
        print(cst, 'cst')
        print(mfess, mfe)
        print(efess, efe)
        print(cfess, cfe)
        print(freq, diversity)

        hr('RNAfold with enforce cst')
        print(cst, 'cst')
        print(mfess_enforce, mfe_enforce)
        print(efess_enforce, efe_enforce)
        print(cfess_enforce, cfe_enforce)
        print(freq_enforce, diversity_enforce)

        print # for space
        df = df.append({
            'id': s.id,
            'seq': s.seq,
            'ss': s.ss,
            'distance': dist,
            'foldability': foldability,
            'eval' : eval,
            'mcsym': ss[0],
            'mcsym ss': ss[1],
            'mcsym comment': ss[2],
            'length': length,

            'mfe' : mfe,
            'mfess' : mfess,

            'efe' : efe,
            'efess' : efess,

            'cfe' : cfe,
            'cfess' : cfess,
            'freq' : freq,
            'diversity' : diversity,

            'mfe_enforce' : mfe_enforce,
            'mfess_enforce' : mfess_enforce,
            'efe_enforce' : efe_enforce,
            'efess_enforce' : efess_enforce,
            'cfe_enforce' : cfe_enforce,
            'cfess_enforce' : cfess_enforce,
            'freq_enforce' : freq_enforce,
            'diversity_enforce' : diversity_enforce,

            }, ignore_index=True)

        if args.dev:
            break

    print(df)
    df.to_csv(args.output)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""rna_simrna_lowest.py - get lowest energy frames out of a SimRNA trajectory file

This code uses heavily the SimRNATrajectory class. Be default 100 lowest energy frames is exported.
"""

from __future__ import print_function
from rna_tools.tools.simrna_trajectory.simrna_trajectory import SimRNATrajectory
import argparse


def get_parser():
    parser = argparse.ArgumentParser(
        description=__doc__, formatter_class=argparse.RawDescriptionHelpFormatter)
    parser.add_argument('-n', '--nstruc', help="SimRNA trafl file", type=int, default=100)
    parser.add_argument('trafl', help="SimRNA trafl file")
    return parser


if __name__ == '__main__':
    parser = get_parser()
    args = parser.parse_args()

    s = SimRNATrajectory()
    fn = args.trafl

    s.load_from_file(fn, top_level=True)

    # for f in s.frames:
    #    print f

    sorted_frames = s.sort(inplace=False)

    for c, f in enumerate(sorted_frames[:args.nstruc]):
        print(c + 1, f)
        # print f.header
        # print f.coords

    s2 = SimRNATrajectory()
    s2.load_from_list(sorted_frames[:args.nstruc])
    s2.plot_energy(fn.replace('.trafl', '.png'))
    s2.save(fn.replace('.trafl', '') + '_top' + str(args.nstruc) + '.trafl')



#!/usr/bin/python
# -*- coding: utf-8 -*-

"""rna_clanstix - a tool for visualizing RNA 3D structures based on pairwise structural similarity with Clans.

We hacked Clans thus instead of BLAST-based distances between sequences, you can analyze distances between structures described as p-values of rmsd (based on the method from the Dokholyan lab.)

Quickref::

      rna_clanstix.py --groups-auto 10 --color-by-homolog --shape-by-source  thf_ref_mapping_pk_refX.txt input2.clans

Running Clans:
To run CLANS you need to have Java 1.4 or better installed (java can be downloaded HERE). For full functionality you will also need the NCBI BLAST,PSI-BLAST and formatdb executables (NCBI). For command line parameters and basic help please refer to the README file.
(source: http://www.eb.tuebingen.mpg.de/research/departments/protein-evolution/software/clans.html)

.. image:: ../../rna_tools/tools/clanstix/doc/yndSrLTb7l.gif

The RMSDs between structures are converted into p-values based on the method from the Dokholyan lab or some hacky way developed by mmagnus .

Color groups
---------------------------------------
You can color your groups:

.. image:: ../../rna_tools/tools/clanstix/doc/rna_clanstix.png

To get colors, run a cmd like this::

   rna_clastix.py rnapz17_matrix_farfar_HelSeedCst.txt --groups 20:seq1+20+20+20+20+20+20:seq10

where with the ``+`` sign you separate groups. Each group has to have a number of structures. Optionally it can have a name, e.g., ``20:seq1``, use ``:`` as a separator. If a provided name is ``native`` then this group will be shown as starts.

Get inspiration for more colors (http://www.rapidtables.com/web/color/RGB_Color.htm)

How to use ClanstixRNA?
----------------------------------------

1. Get a matrix of distances, save it as e.g. matrix.txt (see Comment below)
2. run ClanstixRNA on this matrix to get an input file to Clans (e.g. clans_rna.txt)::

     rna_clanstix.py test_data/matrix.txt # clans.input will be created by default

3. open CLANS and click File -> Load run and load clans_run.txt
4. You're done! :-)

Comment: To get this matrix you can use for example another tool from the rna-pdb-tools packages::

     rna_calc_rmsd_all_vs_all.py -i rp18 -o rp18_rmsd.csv
     rna_clastix.py --groups 1:native+5:3dRNA+
           5:Chen+3:Dokh+5:Feng+5:LeeASModel+
           5:Lee+5:RNAComposer+10:RW3D+5:Rhiju+
           1:YagoubAli+3:SimRNA  rp18_rmsd.csv clans.in

     rna_clastix.py --groups 100+100+100+100+100+100+100+100+100+100+1:native  rp18_rmsd.csv

where ``rp18`` is a folder with structure and ``rp18_rmsd.csv`` is a matrix of all-vs-all rmsds.

.. image:: ../../rna_tools/tools/clanstix/doc/rp18_clanstix.png

Hajdin, C. E., Ding, F., Dokholyan, N. V, & Weeks, K. M. (2010). On the significance of an RNA tertiary structure prediction. RNA (New York, N.Y.), 16(7), 1340–9. doi:10.1261/rna.1837410

An output of this tool can be viewed using CLANS.

Frickey, T., & Lupas, A. (2004). CLANS: a Java application for visualizing protein families based on pairwise similarity. Bioinformatics (Oxford, England), 20(18), 3702–4. doi:10.1093/bioinformatics/bth444
"""

import argparse
import rna_tools.tools.rmsd_signif.rnastruc_pred_signif as pv
import numpy as np
import math
import logging
import time

logging.basicConfig(level=logging.INFO,
                format='%(message)s',
                datefmt='%m-%d %H:%M',
                filename='rna_clanstix.log',
                filemode='w')

console = logging.StreamHandler()
console.setLevel(logging.INFO)
formatter = logging.Formatter('%(message)s')
console.setFormatter(formatter)
logging.getLogger('').addHandler(console)

log = logging.getLogger()
log.setLevel(logging.INFO)


class RNAStructClans:
    """Clans run.

    Usage::

        >>> f = open('matrix.txt')
        >>> ids = f.readline().replace('#','').split()
        >>> c = RNAStructClans(n=len(ids)) # 200?
        >>> c.add_ids(ids)
        >>> c.dist_from_matrix(f)
        >>> print(c.txt)
    """

    def __init__(self, n=10, dotsize=10):
        self.n = n
        self.comment = ''
        #cluster2d=false
        self.txt = """sequences=%i
<param>
maxmove=0.1
pval=%s
usescval=false
complexatt=true
cooling=1.0
currcool=1.0
attfactor=10.0
attvalpow=1
repfactor=10.0
repvalpow=1
dampening=1.0
minattract=1.0
cluster2d=true
blastpath=''
formatdbpath=''
showinfo=false
zoom=0.9
dotsize=%s
ovalsize=10
groupsize=4
usefoldchange=false
avgfoldchange=false
colorcutoffs=0.0;0.1;0.2;0.3;0.4;0.5;0.6;0.7;0.8;0.9;
colorarr=(230;230;230):(207;207;207):(184;184;184):(161;161;161):(138;138;138):(115;115;115):(92;92;92):(69;69;69):(46;46;46):(23;23;23):
</param>""" % (n, args.pvalue, str(dotsize))

    def add_ids(self, ids):
        t = '\n<seq>\n'
        for i in ids:
            t += '>' + i + '\n'
            t += 'X' + '\n'
        t += '</seq>\n'
        if len(ids) != self.n:
            # print 'n != ids'
            raise Exception('n != ids')
        self.txt += t

    def dist_from_matrix(self, lines,pmat=['0'],use_pv=False, debug=False):
        if debug:
            print('Everything but the dists are generated. Use it to edit the original clans input file.')
            return # for some hardcore debugging ;-)
        t = '\n<hsp>\n'
        myp = ''
        c = 0
        c2 = 0
        for l in lines:
            for rmsd in l:
            #for rmsd in l.split():
                if c != c2:
                    if use_pv:
                        dist = pv.get_p_value(rmsd, 1 * 38)[0]  # r.get_rmsd_to(r2), 3)
                    else:
                        # 1e-06 10-1 = 9 10-10 0
                        dist = '1.0E-' + str(max - int(float(rmsd)))
                    myp += ' ' + str(dist)
                    t += str(c) + ' ' + str(c2) + ':' + str(dist) + '\n'
                else :
                    myp += ' ' + '0.0'
                c2 += 1
            myp += '\n'
            c2 = 0
            c += 1

        t += '</hsp>\n'

        self.comment = '# max: %f min (non-zero): %f\n' % (max, min)
        self.comment += '# 1/4 ' + str((max - min) / 4) + ' ' + str(round((max - min) / 4, 0)) + '\n'
        self.comment += '# 1/2 ' + str((max - min) / 2) + ' ' + str(round((max - min) / 2, 0)) + '\n'
        self.comment += '# 1/3 ' + str(((max - min) / 4 ) * 3 ) + ' ' + str(round(((max - min) / 4) * 3, 0)) + '\n'
        for i in range(1,20):
            self.comment += '# connected points with RMSD lower than %iA 1.0E-%i\n' % (i, math.ceil(matrix.max()) - i)

        if pmat[0] == 'True':
            with open('pMatrix.txt', 'w') as f:
                f.write(myp)
        return t

def check_symmetric(a, rtol=1e-05, atol=1e-08):
    """
    https://stackoverflow.com/questions/42908334/checking-if-a-matrix-is-symmetric-in-numpy
    """
    return np.allclose(a, a.T, rtol=rtol, atol=atol)


def get_parser():
    parser = argparse.ArgumentParser(description=__doc__,
                                     formatter_class=argparse.RawDescriptionHelpFormatter)
    parser.add_argument('matrixfn', help="matrix")

    parser.add_argument('--groups-auto', help="define into how many groups make automatically", type=int)

    parser.add_argument('--color-by-homolog', help="color the same homolog in the same way", action='store_true')
    parser.add_argument('--one-target', help="color the same homolog in the same way", action='store_true')

    parser.add_argument('--shape-by-source', help="shape points based on source, SimRNA vs Rosetta (Rosetta models have 'min' in name'",
                        action='store_true')

    parser.add_argument('--debug', action='store_true')

    parser.add_argument('--size', type=int, default=8)

    parser.add_argument('--dont-calc', action='store_true', help="A simple and dirty trick to get "
                        "generates everything but the main distances from the matrix"
                        "useful if you want to run the script to generate different settings, such as"
                        "colors, groups etc. Run the script and then replace parts of the original "
                        "file with the matrix")

    parser.add_argument('--groups', help="groups, at the moment up to 7 groups can be handle"
                        "--groups 1:native+100:zmp+100:zaa+100:zbaa+100:zcp+100:znc"
                        "--groups 1:native+100:nats+100:hom1+100:hom2+100:hom3+100:hom4"
                        "native will light green"
                        "zmp will be forest green"
                        "(easy to be changed in the future)")

    parser.add_argument('--use-pvalue', action='store_true', help="")

    parser.add_argument('--pvalue', default="1.0E-15", help="set p-value for clans.input, default: 1.0E-15")

    parser.add_argument('--output', help="input file for clans, e.g., clans.input", default="clans.input")
    parser.add_argument('--outpmatrix', type=int, default=0, help="If 1, p value matrix will be output in pmatrix.txt file,"
                        "If 0, no p matrix output file")
    return parser

# main
if __name__ == '__main__':
    parser = get_parser()
    args = parser.parse_args()
    debug = args.debug  # as a short cut later
    outpmatrix=args.outpmatrix
    f = open(args.matrixfn)
    # OK, check if there is a header = the line with '#'
    headers = f.readline()
    if headers.strip().startswith('#'):
        # if yes, then split remove # and split into lists
        ids = headers.replace('#', '').split()
    else:
        # if no, then make a list form [0, # of items in the first line]
        ids = [str(i) for i in range(0, len(headers.split()))]
        f.seek(0)

    logging.info(time.strftime("%Y-%m-%d %H:%M:%S"))

    rmsds = []
    for l in f:
            #for rmsd in map(float,l.split()):  # map(float, s.split())
			rmsds.append(map(float,l.split()))
    #print rmsds
    # warning:
    # eh, this matrix is not really used by clanstix main engine
    # it's used to calc min and max value of a matrix
    matrix = np.loadtxt(args.matrixfn)
    global max
    max = int(math.floor(matrix.max()))
    global min
    min = matrix[matrix>0].min()
    if check_symmetric(matrix):
        if args.debug: print('Matrix is symmetrical! ', matrix.shape)
    else:
        raise Exception('Matrix is not symmetrical! Check your matrix', matrix.shape)

    # keep dot quite big by default,
    # but if you use dotsize then keep this one 0
    dotsize =  8
    if args.groups_auto or args.groups:
        dotsize =  0
    print 1
    c = RNAStructClans(n=len(ids), dotsize=dotsize)  # 200?
    c.add_ids(ids)
    if debug:
        print('dist_from_matrix...')
    #pool = mp.Pool(mp.cpu_count())
    pool = ProcessingPool(mp.cpu_count()) #nodes=4)
    #results = pool.starmap(c.dist_from_matrix, [(row, 4, 8) for row in data])
    x=pool.map(c.dist_from_matrix,[rmsds], [str(outpmatrix)])
    #x=parmap.map(c.dist_from_matrix,[rmsds],outpmatrix,args.use_pvalue, args.dont_calc)
    pool.close()
    print 2
    s=''
    for i in x:
        s=s.join(i)
    if debug:
        print('process the matrix')
    #
    # DEFINE GROUPS
    #
    # 1+20+20+20+20+20
    seqgroups = ''
    #colors = ['0;255;102;255', # ligthgreen 1
    #          '0;102;0;255', # forest 2
    # A list of colors used later ....
    colors = [
              '255;102;102;255', # red 3
              '51;51;255;255', # blue 4
              '0;255;255;255', # light blue +1
              '180;38;223;255', # violet 5
              '64;64;64;255', # grey 6
              '255;128;0;255', # orange 7
              '128;0;128;255', # purple 8
              '0;128;128;255', # Teal 9
              '128;0;0;255', # maroon 10
              '0;255;255;255',  # cyjan
              '240;230;140;255', #khaki
              '210;105;30;255', # chocolate
               ]

    # This is pretty much the same list as above, but in here I have more distinguishable colors,
    # as I should be used with less number of groups
    colors_homologs = [
              '255;102;102;255', # red 3
              '51;51;255;255', # blue 4
              '64;64;64;255', # grey 6
              '128;0;128;255', # purple 8
              '128;0;0;255', # maroon 10
              '0;255;255;255',  # cyjan
              '237;41;57;255', # red
              #'210;105;30;255', # chocolate
               ]

    # OK, this is the trick
    # if args.groups_auto is on, then you built args.groups automatically,
    # and then just simply run the next if as it was args.groups in the arguments
    # pretty cool ;-)
    if args.groups_auto:
        # arsg_groups = ''
        from collections import OrderedDict
        # collect groups
        #groups = []
        groups = OrderedDict()
        for i in ids:
            #
            # collect homologs gmp_
            # simrna and farna

            group_name = i[:args.groups_auto]
            if group_name in groups:
                groups[group_name] += 1
            else:
                groups[group_name] = 1
        groups_str = ''
        for g in groups:
            groups_str += str(groups[g]) + ':' + g + '+'
        #
        # this is the trick, you
        args.groups = groups_str[:-1]
        print(groups_str)
        # change this to get 1:hccp+10:zmp+10:xrt

    if args.groups:
        # this is a manual way how to do it
        groups = args.groups.split('+')
        seqgroups = '<seqgroups>\n'
        curr_number = 0
        homologs_colors = {}
        if args.debug: print(args.groups)
        for index, group in enumerate(groups):
            # parse groups
            # type and size will be changed for native
            size = args.size
            dottype = 0
            color = '' # use for diff color selection if tar
            if ':' in group:
                nstruc, name = group.split(':')
                if name == 'native':
                    dottype = 8
                    size = 20
                    color = '0;128;128;255' # olive

                if 'solution' in name:
                    dottype = 8
                    size = 30  # solution is bigger
                    color = '0;128;128;255' # olive

                if args.one_target: # target is target, dont distinguish it
                    if 'tar' in name and 'tar_min' not in name: # SimRNA
                        dottype = 7
                        size = size # 7  # size of SimRNA reference seq points
                        color = '0;255;102;255' # ligthgreen 1

                    if 'tar_min' in name:
                        dottype = 9
                        size = size # 7 # size of Rosetta reference seq points
                        color = '0;102;0;255' # forest 2
                else:
                    if 'tar' in name: # SimRNA
                        size = size #8  # size of SimRNA reference seq points
                        dottype = 0
                        color = '0;255;102;255' # ligthgreen 1

                if args.shape_by_source:
                    # Rosetta models are diamond now
                    if 'min' in name:
                        dottype = 2

                # color by homolog
                if args.color_by_homolog:
                    # 10:gxx_6bd266+10:gxx_min.ou+10:gbaP_d2b57+10:gbaP_min.o+10:gbx_00de79+10:gbx_min.ou+10:gapP_d9d22+10:gapP_min.o+10:gmp_faa97e+10:gmp_min.ou
                    tmp = name.split('_')
                    homolog_name = tmp[0]
                    if debug: 'homolog_name', homolog_name
                    # ignore tar and solution, their colors are defined above ^
                    ## [mm] simrna5x100farna5x100$ git:(master) ✗ rna_clastix.py --groups-auto 8 --color-by-homolog --shape-by-source rp14sub_ref_mapping_refX.txt input2.clans --debug
                    ## 2019-01-09 12:23:21
                    ## 100:tar_min.+100:tar_rp14+1:solution+100:cy2_min.+100:cy2_r14a+100:aj6_min.+100:aj6_r14a
                    ## {'cy2': '210;105;30;255'}
                    ## {'cy2': '210;105;30;255'}
                    ## {'cy2': '210;105;30;255', 'aj6': '0;255;255;255'}
                    ## {'cy2': '210;105;30;255', 'aj6': '0;255;255;255'}
                    ## # max: 18.000000 min (non-zero): 0.810000
                    if homolog_name == 'tar' or homolog_name.startswith('solution'):
                        pass
                    else:
                        if homolog_name in homologs_colors:
                            color = homologs_colors[homolog_name]
                        else:
                            homologs_colors[homolog_name] = colors_homologs.pop()
                            color = homologs_colors[homolog_name]
                        if debug: print(homologs_colors)
            else:
                nstruc = group
                name = 'foo'

            # color is set in the args.color_by_homologs

            # craft seqgroups
            seqgroups += "name=%s\n" % name
            seqgroups += "type=%i\n" % dottype
            # color hack
            if color:  # this color is fixed for native right now
                seqgroups += "color=%s\n" % color
            else:
                seqgroups += "color=%s\n" % colors[index]

            # color hack
            seqgroups += "size=%i\n" % size
            seqgroups += "hide=0\n"
            if debug:
                print('name: ' + name + ' ' + colors[index])
            # get numbers - convert nstruc into numbers in Clans format 0;1; etc.
            # remember: it starts from 0
            # --groups 1:hccp+10:zmp+10:xrt
            # hccp
            # [0]
            # zmp
            # [1, 2, 3, 4, 5, 6, 7, 8, 9, 10]
            # xrt
            # [11, 12, 13, 14, 15, 16, 17, 18, 19, 20]
            numbers = range(curr_number, curr_number + int(nstruc))  # 0, 1
            curr_number = curr_number + int(nstruc)
            seqgroups += "numbers=%s;\n" % ';'.join([str(number) for number in numbers])
        seqgroups += '</seqgroups>\n'
    #print c.txt
    with open(args.output, 'w') as f:
        f.write(c.txt)
        #for i_x in y:
        f.write(s)
        #f.write(str(x))
        f.write(seqgroups)
        f.write(c.comment)
    print(c.comment)
    logging.info(time.strftime("%Y-%m-%d %H:%M:%S"))
    if debug:
        print(seqgroups)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Python parser to 3dna <http://x3dna.org/>.

Installation::

  # install the code from http://forum.x3dna.org/downloads/3dna-download/
  Create a copy of the rna_x3dna_config_local_sample.py (remove "_sample") present in rna-tools/rna_tools/tools/rna_x3dna folder.
  Edit this line :
  BINARY_PATH = <path to your x3dna-dssr file>
  matching the path with the path of your x3dna-dssr file.
  e.g. in my case: BINARY_PATH = ~/bin/x3dna-dssr.bin

For one structure you can run this script as::

    [mm] py3dna$ git:(master) ✗ ./rna_x3dna.py test_data/1xjr.pdb
    test_data/1xjr.pdb
    >1xjr nts=47 [1xjr] -- secondary structure derived by DSSR
    gGAGUUCACCGAGGCCACGCGGAGUACGAUCGAGGGUACAGUGAAUU
    ..(((((((...((((.((((.....))..))..))).).)))))))

For multiple structures in the folder, run the script like this::

    [mm] py3dna$ git:(master) ✗ ./rna_x3dna.py test_data/*
    test_data/1xjr.pdb
    >1xjr nts=47 [1xjr] -- secondary structure derived by DSSR
    gGAGUUCACCGAGGCCACGCGGAGUACGAUCGAGGGUACAGUGAAUU
    ..(((((((...((((.((((.....))..))..))).).)))))))
    test_data/6TNA.pdb
    >6TNA nts=76 [6TNA] -- secondary structure derived by DSSR
    GCGGAUUUAgCUCAGuuGGGAGAGCgCCAGAcUgAAgAPcUGGAGgUCcUGUGtPCGaUCCACAGAAUUCGCACCA
    (((((((..((((.....[..)))).((((.........)))).....(((((..]....))))))))))))....
    test_data/rp2_bujnicki_1_rpr.pdb
    >rp2_bujnicki_1_rpr nts=100 [rp2_bujnicki_1_rpr] -- secondary structure derived by DSSR
    CCGGAGGAACUACUG&CCGGCAGCCU&CCGGAGGAACUACUG&CCGGCAGCCU&CCGGAGGAACUACUG&CCGGCAGCCU&CCGGAGGAACUACUG&CCGGCAGCCU
    [[[[(((.....(((&{{{{))))))&(((((((.....(.(&]]]]).))))&[[[[[[......[[[&))))]]].]]&}}}}(((.....(((&]]]]))))))

"""
import re
import argparse

from subprocess import Popen, PIPE
from os import remove, path, readlink

PATH = path.abspath(__file__)
if path.islink(PATH):
    PATH = path.dirname(readlink(PATH))
else:
    PATH = path.dirname(path.abspath(__file__))

from rna_tools.tools.rna_x3dna.rna_x3dna_config import BINARY_PATH


class x3DNAMissingFile(Exception):
    pass


def get_parser():
    parser = argparse.ArgumentParser(
        description=__doc__, formatter_class=argparse.RawDescriptionHelpFormatter)
    parser.add_argument('-c', '--compact',  action='store_true')
    parser.add_argument('-l', '--show-log',  action='store_true', help="show full log")
    parser.add_argument('files', help='file', nargs='+')
    return parser


class x3DNA(object):

    """
    Atributes:

     **curr_fn**
     **report**

    """

    def __init__(self, pdbfn, show_log=False):
        """Set self.curr_fn based on pdbfn"""
        self.curr_fn = pdbfn
        self.run_x3dna(show_log)
        self.clean_up()

    def __get_report(self):
        """Off right now. Run find_pair

        ..warning:: To get modification use get_modifications()
        """
        cmd = BINARY_PATH_FP + ' ' + self.curr_fn + ' stdout | analyze stdin'  # hack
        out = Popen([cmd], stderr=PIPE, stdout=PIPE, shell=True)

        stdout = out.stdout.read()
        outerr = out.stderr.read()

        text = ''
        for l in outerr.split('\n'):
            if l.startswith('Time used') or l.startswith('..') or l.startswith('handling') or l.startswith('uncommon residue'):
                continue
            if l.strip():
                text += l + '\n'
        return text.strip()

    def get_modifications(self):
        """Run find_pair to find modifications.
        """
        cmd = BINARY_PATH_FP + ' -p ' + self.curr_fn + ' /tmp/fpout'  # hack
        out = Popen([cmd], stderr=PIPE, stdout=PIPE, shell=True)

        outerr = out.stderr.read()

        text = ''
        for l in outerr.split('\n'):
            if l.startswith('uncommon residue'):
                text += l.replace('uncommon residue ', '') + '\n'
        return text.strip()

    def run_x3dna(self, show_log=False):
        """
        """
        cmd = BINARY_PATH + ' -i=' + self.curr_fn
        out = Popen([cmd], stderr=PIPE, stdout=PIPE, shell=True)

        stdout = str(out.stdout.read())
        outerr = str(out.stderr.read())

        f = open('py3dna.log', 'w')
        f.write(cmd + '\n' + stdout)
        if args.show_log:
            print(stdout)
        f.close()

        if outerr.find('does not exist!') > -1:  # not very pretty
            raise Py3DNAMissingFile
        if outerr.find('not found') > -1:  # not very pretty
            raise Exception('x3dna not found!')

        rx = re.compile('no. of DNA/RNA chains:\s+(?P<no_DNARNAchains>\d+)\s').search(stdout)
        if rx:
            no_of_DNARNA_chains = int(rx.group('no_DNARNAchains'))
            msg = 'py3dna::no of DNARNA chains'
            self.report = msg + '\n' + msg + '\n'  # hack!
        else:
            raise Exception('no_of_DNARNA_chains not found')

        if no_of_DNARNA_chains:
            self.report = stdout.strip()

    def get_ion_water_report(self):
        """@todo
File name: /tmp/tmp0pdNHS
    no. of DNA/RNA chains: 0 []
    no. of nucleotides:    174
    no. of waters:         793
    no. of metals:         33 [Na=29, Mg=1, K=3]
        """
        pass

    def clean_up(self, verbose=False):
        files_to_remove = [
            'dssr-helices.pdb',
            'dssr-pairs.pdb',
            'dssr-torsions.dat',
            'dssr-hairpins.pdb',
            'dssr-multiplets.pdb',
            'dssr-stems.pdb',
            'dssr-Aminors.pdb',
            'hel_regions.pdb',
            'bp_order.dat',
            'bestpairs.pdb',
        ]

        for f in files_to_remove:
            try:
                remove(f)
            except OSError:
                if verbose:
                    print('can not remove %s' % f)

    def get_seq(self):
        """Get sequence.

        Somehow 1bzt_1 x3dna	UCAGACUUUUAAPCUGA, what is P?
        P -> u
        """
        return self.report.split('\n')[-2].replace('P', 'u').replace('I', 'a')

    def get_secstruc(self):
        """Get secondary structure.
        """
        return open('dssr-2ndstrs.dbn').read().strip()  # self.report.split('\n')[-1]


# name
if __name__ == '__main__':
    if not BINARY_PATH:
        raise Exception(
            'Set up BINARY_PATH in rna_x3dna_config_local.py, .e.g "/Users/magnus/work/opt/x3dna/x3dna-dssr"')

    # get parser and arguments
    parser = get_parser()
    args = parser.parse_args()

    # try:
    #    compact = sys.argv[2]
    #    if compact == '-c':
    #       compact = True
    #
    # except IndexError:
    #    compact = False

    for f in args.files:
        if args.compact:
            p = x3DNA(f)
            print((f, p.get_secstruc()))
        else:
            print(f)

            p = x3DNA(f, args.show_log)

            #s = p.get_seq()
            # print s

            s = p.get_secstruc()
            print(s)
